/**
 * 
 */
package org.jewel.core;

import java.lang.reflect.Method;

import org.jewel.annotations.Controller;
import org.jewel.annotations.ControllerScope;

/**
 * <p>Classe que cont la informaci d'una acci: el mtode que representa l'acci
 * i la classe controladora on es troba</p>
 * 
 * @author Juanma
 * @version 1.0
 * 
 */
public class ActionMap {
	
	/**
	 * <p>Nom del controlador (atribut <i>name</i> de l'anotaci {@link Controller})</p>
	 */
	
	private String controllerName;
	/**
	 * <p>Classe controladora</p>
	 */
	private Class<?> controller;
	
	/**
	 * <p>Mtode que representa l'acci</p>
	 */
	private Method method;

	/**
	 * <p>Constructor</p>
	 * 
	 * @param controllerName Nom del controlador
	 * @param controller Classe controladora
	 * @param method Mtode de l'acci
	 */
	public ActionMap(String controllerName, Class<?> controller, Method method) {
		this.controllerName = controllerName;
		this.controller = controller;
		this.method = method;
	}

	
	/**
	 * <p>Retorna <i>true</i> si el controlador s de sessi</p>
	 * @return <i>true</i> Si s un controlador de sessi. <i>false</i> en cas contrari.
	 */
	public boolean isSessionScope() {
		Controller aController = controller.getAnnotation(Controller.class);
		return (aController != null ? aController.scope().equals(ControllerScope.SESSION) : false);
	}
	
	
	/**
	 * <p>Retorna <i>true</i> si el controlador s de petici</p>
	 * @return <i>true</i> Si s un controlador de petici. <i>false</i> en cas contrari.
	 */
	public boolean isRequestScope() {
		Controller aController = controller.getAnnotation(Controller.class);
		return (aController != null ? aController.scope().equals(ControllerScope.REQUEST) : true); // Per defecte s de tipus Request
	}
	
	
	// Getters
	//
	public String getControllerName() {
		return controllerName;
	}
	public Class<?> getController() {
		return controller;
	}
	public Method getMethod() {
		return method;
	}
	
}
