/**
 * 
 */
package org.jewel.core;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * <p>Configuraci de la internacionalitzaci a l'aplicaci.</p>
 * <p>Segueix el patr Singleton, pel que noms hi ha una instancia de la classe
 * a tota l'aplicaci</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class LocaleConfiguration {

	/**
	 * Llistat d'idiomes disponibles a l'aplicaci
	 */
	private List<Locale> availableLocales;
	
	/**
	 * Idioma per defecte de l'aplicaci
	 */
	private Locale defaultLocale;
	
	/**
	 * Nom base dels arxius de propietats on es defineixen els 
	 * missatges internacionalitzats.
	 */
	private String messagesFile;
	
	/**
	 * Nom del parmetre GET que s'utilitzar per indicar un canvi
	 * d'idioma
	 */
	private String changeParameter;
	
	/**
	 * <p>Nom de la variable que s'utilitzar a les vistes per accedir
	 * als missatges internacionalitzats.</p>
	 * <p>Per exemple, si t el valor 'msgs' s'accedir:<br/>
	 * "#{msgs['titol_aplicacio'] }"</p>
	 */
	private String messagesVariable;
	
	
	public LocaleConfiguration() {
		this.availableLocales = new ArrayList<Locale>();
	}
	
	
	/**
	 * <p>Afegeix l'idioma per defecte de l'aplicaci</p>
	 * <p>S'utilitza al configurar l'aplicaci parsejant l'arxiu <i>jewel-config.xml</i></p>
	 * 
	 * @param language Nom que identifica l'idioma
	 */
	public void addDefaultLanguage(String language) {
		this.defaultLocale = new Locale(language);
		addLanguage(language);
	}
	
	
	/**
	 * <p>Afegeix un idioma a l'aplicaci</p>
	 * <p>S'utilitza al configurar l'aplicaci parsejant l'arxiu <i>jewel-config.xml</i></p>
	 * 
	 * @param language Nom que identifica l'idioma
	 */
	public void addLanguage(String language) {
		Locale locale = new Locale(language);
		if (!this.availableLocales.contains(locale)){
			this.availableLocales.add(locale);
		}
	}
	
	
	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer("********* Locale Configuration *********\n");
		sb.append("Available Locales: ");
		for (Locale l : availableLocales) {
			sb.append(l.getLanguage() + ", ");
		}
		sb.append(defaultLocale);
		sb.append("\nDefault locale: " + defaultLocale + "\n");
		sb.append("Messages file: " + messagesFile + "\n");
		sb.append("Change language parameter: " + changeParameter + "\n");
		sb.append("Messages JSP variable: " + messagesVariable + "\n");
		return sb.toString();
	}
	
	
	// Getters / Setters
	//
	public List<Locale> getAvailableLocales() {
		return availableLocales;
	}
	public void setAvailableLocales(List<Locale> availableLocales) {
		this.availableLocales = availableLocales;
	}
	public Locale getDefaultLocale() {
		return defaultLocale;
	}
	public void setDefaultLocale(Locale defaultLocale) {
		this.defaultLocale = defaultLocale;
	}
	public String getMessagesFile() {
		return messagesFile;
	}
	public void setMessagesFile(String messagesFile) {
		this.messagesFile = messagesFile;
	}
	public String getChangeParameter() {
		return changeParameter;
	}
	public void setChangeParameter(String changeParameter) {
		this.changeParameter = changeParameter;
	}
	public String getMessagesVariable() {
		return messagesVariable;
	}
	public void setMessagesVariable(String messagesVariable) {
		this.messagesVariable = messagesVariable;
	}
	
}
