/**
 * 
 */
package org.jewel.core;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.jewel.util.JewelConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Filtre que proporciona Jewel per facilitar la internacionalitzaci de l'aplicaci.
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class LocaleFilter {

	private static final Logger _log = LoggerFactory.getLogger(LocaleFilter.class);
	
	
	/**
	 * <p>Mtode que s'executa abans de l'acci.</p>
	 * <p>Mira si s'ha canviat l'idioma i el canvia a l'atribut de sessi. Tamb posa
	 * disponible al context de la petici el paquet de recursos amb els missatges en 
	 * l'idioma actual</p>
	 * 
	 * @param ctx RequestContext
	 */
	public void preAction(RequestContext ctx) {
		_log.info("LocaleFilter preAction");
		ApplicationConfiguration config = ApplicationConfiguration.getInstance();

		Locale locale = null;
		if (ctx.getNewLocale() != null && !"".equals(ctx.getNewLocale())) {
			// Canviem l'idioma
			locale = new Locale(ctx.getNewLocale());
			if (!config.getLocaleConfig().getAvailableLocales().contains(locale)) {
				_log.error("No s'ha trobat l'idioma '" + ctx.getNewLocale() + "' entre els idiomes disponibles a l'aplicaci.");
				locale = config.getLocaleConfig().getDefaultLocale();
			}
		}
		else if (ctx.getRequest().getSession().getAttribute(JewelConstants.CURRENT_LOCALE_KEY) != null) {
			// Obtenim l'idioma actual de la sessi
			String sLocale = (String) ctx.getRequest().getSession().getAttribute(JewelConstants.CURRENT_LOCALE_KEY);
			locale = new Locale(sLocale);
		}
		else {
			// No hi ha cap idioma a la sessi. Posam l'idioma per defecte de l'aplicaci
			locale = config.getLocaleConfig().getDefaultLocale();
		}
		
		if (locale != null) {
			ctx.setCurrentLocale(locale);
			ctx.getRequest().getSession().setAttribute(JewelConstants.CURRENT_LOCALE_KEY, locale.toString());
			
			try {
				ResourceBundle rb = ResourceBundle.getBundle(config.getLocaleConfig().getMessagesFile(), locale);
				ctx.setResourcebundle(rb);
				// Put attribute ResourceBundle
				if (rb != null) {
					ctx.getRequest().setAttribute(config.getLocaleConfig().getMessagesVariable(), rb);
				}
			}
			catch (MissingResourceException e) {
				_log.error("No s'ha trobat cap arxiu de missatges en l'idioma " + ctx.getCurrentLocale() + ".", e);
			}
		}
	}

}
