/**
 * 
 */
package org.jewel.core;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jewel.util.JewelConstants;

/**
 * <p>Classe que implementa el patr Object Context.</p>
 * <p>Cont les dades d'una petici</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class RequestContext {

	/**
	 * URL de la petici
	 */
	private String url;
	
	/**
	 * Identificador de l'idioma canviat
	 */
	private String newLocale;
	
	/**
	 * Idioma actual
	 */
	private Locale currentLocale;
	
	/**
	 * Paquet de recursos amb els missatges de l'aplicaci
	 * en l'idioma actual.
	 */
	private ResourceBundle resourceBundle;
	
	/**
	 * Mostra per validar el reenviament de formularis
	 */
	private String token;
	
	/**
	 * Parmetres de la petici enviats
	 */
	private Map<String, Object> params;
	
	/**
	 * Controladors de sessi enmagatzemats en sessi.
	 */
	private Map<String, Object> sessionControllers;
	
	/**
	 * Indica si s una petici AJAX
	 */
	private boolean ajax;
	
	private HttpServletRequest req;
	
	
	/**
	 * <p>Crea un objecte {@link RequestContext}</p>
	 * <p>Obt la URL de l'acci, els parmetres, el locale (si escau), el token (si escau)
	 * i guarda la referncia a la petici i a la resposta</p>
	 * 
	 * @param req Petici javax.servlet.http.HttpServletRequest
	 * @param resp Resposta javax.servlet.http.HttpServletResponse
	 */
	@SuppressWarnings("unchecked")
	public RequestContext(final HttpServletRequest req, final HttpServletResponse resp) {
		this.params = new HashMap<String, Object>();
		
		this.url = req.getRequestURI().replace(req.getContextPath(), "");
		this.req = req;
		
		sessionControllers = (Map<String, Object>) req.getSession().getAttribute(JewelConstants.SESSION_CONTROLLERS_KEY);
		if (sessionControllers == null) {
			sessionControllers = new HashMap<String, Object>();
		}
		
		ApplicationConfiguration config = ApplicationConfiguration.getInstance();
		for (Entry<String, String[]> entry : req.getParameterMap().entrySet()) {
			//
			if (config.isLocaleFilter() && entry.getKey().equals(config.getLocaleConfig().getChangeParameter())) {
				this.newLocale = entry.getValue()[0];
			}
			else if (config.isTokenFilter() && entry.getKey().equals(JewelConstants.CURRENT_TOKEN_KEY)) {
				if (entry.getValue()[0] != null && !"".equals(entry.getValue()[0])){
					this.token = entry.getValue()[0];
				}
			}
			else if (entry.getKey().equals("ajax")) {
				// Indica que s una petici JSON, i per tant no s'ha de validar el token
				this.ajax = Boolean.valueOf(entry.getValue()[0]);
			}
			else {
				if (entry.getValue().length == 1) {
					this.params.put(entry.getKey(), entry.getValue()[0]);
				}
				else if (entry.getValue().length > 0) {
					this.params.put(entry.getKey(), entry.getValue());
				}
			}
		}
	}
	
	
	/**
	 * Afegeix un controlador de sessi al context i el guarda a la sessi
	 * 
	 * @param controllerName
	 * @param controller
	 */
	public void addSessionController(String controllerName, Object controller) {
		this.sessionControllers.put(controllerName, controller);
		req.getSession().setAttribute(JewelConstants.SESSION_CONTROLLERS_KEY, this.sessionControllers);
	}
	
	
	/**
	 * <p>Obt tots els parmetres del context que s'han d'assignar a un controlador. s a dir,
	 * aquells que segueixin el patr 'controllerName.< parmetre >'</p>
	 * 
	 * @param controllerName Nom donat al controlador
	 * @return Conjunt de parmetres del controlador
	 */
	public Map<String, Object> getControllerParameters(String controllerName) {
		Map<String, Object> controllerParams = new HashMap<String, Object>();
		for (Entry<String, Object> param : this.params.entrySet()) {
			if (param.getKey().startsWith(controllerName+".")) {
				controllerParams.put(param.getKey().substring(param.getKey().indexOf(".")+1), param.getValue());
			}
		}
		return controllerParams;
	}
	
	
	// Getters / Setters
	//
	public String getNewLocale() {
		return newLocale;
	}
	public Locale getCurrentLocale() {
		return currentLocale;
	}
	public Object getParameter(String key) {
		return params.get(key);
	}
	public String getToken() {
		return token;
	}
	public String getUrl() {
		return url;
	}
	public boolean isAjax() {
		return ajax;
	}
	public HttpServletRequest getRequest() {
		return req;
	}
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	public Map<String, Object> getSessionControllers() {
		return sessionControllers;
	}
	protected void setResourcebundle(ResourceBundle rb) {
		this.resourceBundle = rb;
	}
	protected void setCurrentLocale(Locale currentLocale) {
		this.currentLocale = currentLocale;
	}
	protected void setNewLocale(String newLocale) {
		this.newLocale = newLocale;
	}
}
