/**
 * 
 */
package org.jewel.core;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.jewel.exceptions.ResultDispatchException;

/**
 * <p>Classe de la qual han d'estendre totes les classes que implementin possibles
 * resultats (TILES, JSP, o altres de personalitzats p.ex: XSL)</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public interface ResultDispatcher {
	
	/**
	 * Mtode per presentar un resultat a partir d'un {@link ModelAndView}
	 * 
	 * @param mav ModelAndView amb el model i la vista
	 * @param req ServletRequest
	 * @param resp ServletResponse
	 * @throws ResultDispatchException Error al presentar la vista
	 */
	public void dispatch(ModelAndView mav, ServletRequest req, ServletResponse resp) throws ResultDispatchException;
	
	/**
	 * Mtode per presentar un error produt durant la petici
	 * 
	 * @param e Excepci a mostrar a la vista
	 * @param errorPage Pgina que mostrar l'error
	 * @param req ServletRequest
	 * @param resp ServletResponse
	 * @throws ResultDispatchException Error al presentar la vista
	 */
	public void dispatchError(Exception e, String errorPage, ServletRequest req, ServletResponse resp) throws ResultDispatchException;
	
}
