/**
 * 
 */
package org.jewel.core;

import javax.servlet.ServletException;

import org.jewel.core.RequestContext;
import org.jewel.exceptions.TokenException;
import org.jewel.util.JewelConstants;

/**
 * Filtre que proporciona Jewel per validar una mostra i evitar el reenviament de formularis.
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class TokenFilter {

	/**
	 * <p>Mtode que s'executa abans de l'acci.</p>
	 * <p>Comprova la mostra d'un formulari i retorna error si no coincideix</p>
	 * 
	 * @param ctx RequestContext
	 * @throws ServletException No coincideix la mostra guardada en sessi amb l'enviada pel formulari
	 */
	public void preAction(RequestContext ctx) throws TokenException {
		if (ctx.getRequest().getMethod().equals("POST") && !ctx.isAjax()) {
			String sessionToken = (String) ctx.getRequest().getSession().getAttribute(JewelConstants.CURRENT_TOKEN_KEY);
			String requestToken = ctx.getToken();
			
			if (sessionToken != null && requestToken != null && !sessionToken.equals(requestToken)) {
				throw new TokenException("Reenviament de formulari!");
			}
		}
	}
	
	
	/**
	 * <p>Mtode que s'executa desprs de l'acci.</p>
	 * <p>Crea una mostra i la guarda en sessi. S'utilitzar a la prxima petici per comprobar 
	 * que no es reenvii el formulari</p>
	 * 
	 * @param ctx RequestContext
	 */
	public void postAction(RequestContext ctx) {
		if (!ctx.isAjax()) {
			ctx.getRequest().getSession().setAttribute(JewelConstants.CURRENT_TOKEN_KEY, "token_" + System.currentTimeMillis());
		}
	}
}
