/**
 * 
 */
package org.jewel.core.dispatchers;

import java.util.Map.Entry;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.jewel.core.ModelAndView;
import org.jewel.core.ResultDispatcher;
import org.jewel.exceptions.ResultDispatchException;
import org.jewel.util.JewelConstants;

/**
 * Classe abstracta que generalitza la presentaci de resultats en JSPs
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public abstract class AbstractJspResultDispatcher implements ResultDispatcher {

	/**
	 * @see org.jewel.core.ResultDispatcher#dispatch(org.jewel.core.ModelAndView, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatch(ModelAndView mav, ServletRequest req,
			ServletResponse resp) throws ResultDispatchException {
		// Posam el model accessible a la vista
		//
		if (mav != null) {
			if (mav.getModel() != null) {
				for (Entry<String, Object> entry : mav.getModel().entrySet()) {
					req.setAttribute(entry.getKey(), entry.getValue());
				}
			}
			if (mav.getMessages() != null) {
				if (mav.getMessages().getErrorMessages() != null && mav.getMessages().getErrorMessages().size() > 0) {
					req.setAttribute(JewelConstants.ERROR_MESSAGES_KEY, mav.getMessages().getErrorMessages());
				}
				if (mav.getMessages().getInfoMessages() != null && mav.getMessages().getInfoMessages().size() > 0) {
					req.setAttribute(JewelConstants.INFO_MESSAGES_KEY, mav.getMessages().getInfoMessages());
				}
			}
		}
	}
	
	
	/**
	 * @see org.jewel.core.ResultDispatcher#dispatchError(java.lang.Exception, java.lang.String, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatchError(Exception e, String errorPage,
			ServletRequest req, ServletResponse resp)
			throws ResultDispatchException {
		req.setAttribute(JewelConstants.EXCEPTION_ERROR_KEY, e);
	}

}
