/**
 * 
 */
package org.jewel.core.dispatchers;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.jewel.core.ModelAndView;
import org.jewel.core.ResultDispatcher;
import org.jewel.exceptions.ResultDispatchException;

import com.google.gson.Gson;

/**
 * Classe que implementa un resultat de tipus Json (per cridades Ajax).
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class JsonResultDispatcher implements ResultDispatcher {

	/**
	 * @see org.jewel.core.ResultDispatcher#dispatch(org.jewel.core.ModelAndView, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatch(ModelAndView mav, ServletRequest req,
			ServletResponse resp) throws ResultDispatchException {
		resp.setContentType("application/json");
		PrintWriter out;
		try {
			out = resp.getWriter();
			Gson gson = new Gson();
			// Convertim el model al model de sortida adequat al tipus de presentaci
			String model = gson.toJson(mav.getModel());
			out.print(model);
			out.flush();
		} catch (IOException e) {
			throw new ResultDispatchException("Error dispatxant el resultat JSON", e);
		}
	}

	
	/**
	 * @see org.jewel.core.ResultDispatcher#dispatchError(java.lang.Exception, java.lang.String, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatchError(Exception error, String errorPage, ServletRequest req,
			ServletResponse resp) throws ResultDispatchException {
		resp.setContentType("application/json");
		PrintWriter out;
		try {
			out = resp.getWriter();
			Gson gson = new Gson();
			// Convertim l'error a JSON
			String model = gson.toJson(error);
			out.print(model);
			out.flush();
		}
		catch (IOException e) {
			throw new ResultDispatchException("Error dispatxant el resultat JSON", e);
		}
	}

}
