/**
 * 
 */
package org.jewel.core.dispatchers;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.jewel.core.ModelAndView;
import org.jewel.core.ResultDispatcher;
import org.jewel.exceptions.ResultDispatchException;

/**
 * Classe que implementa un resultat de tipus JSP.
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class JspResultDispatcher extends AbstractJspResultDispatcher implements ResultDispatcher {

	/**
	 * Implementaci del mtode 'dispatch' per presentar una vista JSP
	 * @see org.jewel.core.ResultDispatcher#dispatch(org.jewel.core.ModelAndView, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatch(ModelAndView mav, ServletRequest req, ServletResponse resp) throws ResultDispatchException {
		// Posa el model i els missatges accessibles des de la vista com atributs.
		super.dispatch(mav, req, resp);
		
		try {
			// Presentam la vista
			RequestDispatcher dispatcher = req.getRequestDispatcher(mav.getView());
			dispatcher.forward(req, resp);
		}
		catch (Exception e) {
			throw new ResultDispatchException("No s'ha pogut presentar la vista " + mav.getView(), e);
		}
	}

	
	/**
	 * Implementaci del mtode 'dispatchError' per presentar una vista JSP pels errors
	 * @see org.jewel.core.ResultDispatcher#dispatchError(java.lang.Exception, java.lang.String, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatchError(Exception error, String errorPage, ServletRequest req,
			ServletResponse resp) throws ResultDispatchException {
		super.dispatchError(error, errorPage, req, resp);
		
		if (errorPage != null) {
			try {
				// Presentam la vista
				RequestDispatcher dispatcher = req.getRequestDispatcher(errorPage);
				dispatcher.forward(req, resp);
			}
			catch (Exception e) {
				throw new ResultDispatchException("No s'ha pogut presentar la vista " + errorPage, e);
			}
		}
		else {
			// L'usuari no proporciona un JSP. Generem la vista
			try {
				resp.setContentType("text/html");
				resp.getWriter().println("<html><head><title>Jewel Error Page</title></head><body>");
				resp.getWriter().println("<h1>S'ha produt un error inesperat a l'aplicaci</h1>");
				resp.getWriter().println("<p>" + error.getMessage() + "</p>");
				resp.getWriter().println("<p>Contacti amb l'administrador de l'aplicaci.</p>");
				resp.getWriter().println("</body></html>");
				resp.getWriter().close();
			} catch (IOException e) {
				throw new ResultDispatchException("No s'ha pogut presentar la vista " + errorPage, e);
			}
		}
	}

}
