/**
 * 
 */
package org.jewel.core.dispatchers;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.tiles.TilesContainer;
import org.apache.tiles.servlet.context.ServletUtil;
import org.jewel.core.ModelAndView;
import org.jewel.core.ResultDispatcher;
import org.jewel.exceptions.ResultDispatchException;

/**
 * Classe que implementa un resultat de tipus Apache Tiles
 * per poder presentar vistes basades en plantilles Tiles.
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class TilesResultDispatcher extends AbstractJspResultDispatcher implements ResultDispatcher {

	/**
	 * Implementaci del mtode 'dispatch' per presentar una vista basada en plantilles Tiles
	 * @see org.jewel.core.ResultDispatcher#dispatch(ModelAndView, ServletRequest, ServletResponse)
	 */
	@Override
	public void dispatch(ModelAndView mav, ServletRequest req, ServletResponse resp) throws ResultDispatchException {
		// Posa el model i els missatges accessibles des de la vista com atributs.
		super.dispatch(mav, req, resp);
		
		// Presentam la vista
		TilesContainer container = ServletUtil.getContainer(req.getServletContext());
		container.render(mav.getView(), req, resp);
	}

	
	/**
	 * @see org.jewel.core.ResultDispatcher#dispatchError(java.lang.Exception, java.lang.String, javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void dispatchError(Exception e, String errorPage,
			ServletRequest req, ServletResponse resp)
			throws ResultDispatchException {
		super.dispatchError(e, errorPage, req, resp);
		
		// Presentam la vista
		TilesContainer container = ServletUtil.getContainer(req.getServletContext());
		container.render(errorPage, req, resp);
	}

}
