/**
 * 
 */
package org.jewel.taglib;

import javax.servlet.jsp.JspException;

import org.jewel.util.ParameterUtil;

/**
 * <p>Classe amb els mtodes i propietats comuns a tots els inputs d'un formulari</p>
 * <p>P.ex: {@link InputTextTag}, {@link InputDateTag}, {@link InputHiddenTag}, {@link InputPasswordTag}, 
 * {@link TextareaTag}, {@link SelectTag}, {@link SelectRadioTag}, ...</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public abstract class AbstractInputTag extends AbstractHtmlTag {

	private static final long serialVersionUID = -3285705487963953820L;
	
	// Atributs de l'etiqueta
	private String value;
	protected boolean required;
	protected String requiredMessage;
	// Atributs propis de l'etiqueta HTML
	protected boolean disabled;
	protected boolean readonly;
	protected Integer size;
	protected Integer tabindex;
	protected String name;
	
	
	protected Object getValue() throws JspException {
		if (this.value == null) return "";
		return ParameterUtil.getExpressionValue(this.value, pageContext);
	}
	
	protected boolean isRequired() {
		return this.required;
	}
	protected String getRequiredMessage() throws JspException {
		Object aux = ParameterUtil.getExpressionValue(this.requiredMessage, pageContext);
		if (aux == null) return "";
		return aux.toString();
	}
	
	protected StringBuffer getParentAttributes() {
		StringBuffer sb = super.getParentAttributes();
		if (name != null)
			sb.append(" name=\"" + name + "\" ");
		if (disabled)
			sb.append(" disabled=\"disabled\" ");
		if (readonly)
			sb.append(" readonly=\"readonly\"");
		if (size != null && size > 0)
			sb.append(" size=\"" + size + "\"");
		if (tabindex != null && tabindex > 0)
			sb.append(" tabindex=\"" + tabindex + "\"");
		return sb;
	}

	
	// Setters
	public void setValue(String value) {
		if (value == null) this.value = "";
		else this.value = value;
	}
	public void setRequired(boolean required) {
		this.required = required;
	}
	public void setRequiredMessage(String requiredMessage) {
		this.requiredMessage = requiredMessage;
	}
	public void setDisabled(boolean disabled) {
		this.disabled = disabled;
	}
	public void setReadonly(boolean readonly) {
		this.readonly = readonly;
	}
	public void setSize(Integer size) {
		this.size = size;
	}
	public void setTabindex(Integer tabindex) {
		this.tabindex = tabindex;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	// Getters
	public Integer getSize() {
		return size;
	}
	public boolean isDisabled() {
		return disabled;
	}
	public boolean isReadonly() {
		return readonly;
	}
	
}
