/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import org.jewel.exceptions.InvalidPathPatternException;
import org.jewel.util.JewelConstants;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;
import org.jewel.util.UrlUtil;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:commandButton&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class CommandButtonTag extends AbstractHtmlTag {

	private static final long serialVersionUID = -9014328704168137751L;

	// Plantilles de les etiquetes HTML
	private static final String BUTTON_TPL = "<input type=\"submit\" value=\"%s\" %s />";
	
	// Atributs de l'etiqueta
	private String action;
	private boolean validateForm = true;
	private String label;
	private boolean tokenCheck = true;
	private Integer tabindex;
	private String name;
	
	private Map<String, String> parametres;
	
	@Override
	public int doStartTag() throws JspException {
		if (pageContext.getAttribute("current_form") == null) throw new JspException("No pots posar un bot Command sense un formulari!");
		
		this.parametres = new HashMap<String, String>();
		
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			String value = (label != null ?  (String) ParameterUtil.getExpressionValue(label, pageContext) : "Submit");
			
			String completeUrl = UrlUtil.getUrl(((HttpServletRequest) pageContext.getRequest()).getContextPath(), action, parametres);
			
			String currentForm = (String) pageContext.getAttribute("current_form");
			
			//if (!ApplicationController.getInstance().validateAction(completeUrl)) throw new JspException("L'acci especificada al bot " + inputName + " no s vlida: " + url);
			
			StringBuffer attributes = new StringBuffer();
			// Insertem el codi Javascript validar i enviar el formulari
			attributes.append(" onclick=\"");
			if (validateForm) {
				attributes.append(" if (!validateForm" + currentForm + "()) return false; ");
			}
			attributes.append("document.forms['" + currentForm + "'].action = '" + completeUrl + "';");
			
			if (!tokenCheck) {
				attributes.append("document.forms['" + currentForm + "'].elements['" + JewelConstants.CURRENT_TOKEN_KEY + "'].value = '';");
			}
			attributes.append("document.forms['" + currentForm + "'].submit();");
			attributes.append("\"");
			
			attributes.append(getParentAttributes());
			
			if (tabindex != null)
				attributes.append(" tabindex=\"" + tabindex + "\" ");
			if (name != null)
				attributes.append(" name=\"" + name + "\" ");
			
			pageContext.getOut().println(String.format(BUTTON_TPL, value, attributes.toString()));
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		catch (InvalidPathPatternException e) {
			throw new JspException("Patr URL invalid al bot " + id + "\n" + e.getMessage());
		}
		
		return EVAL_PAGE; // Seguir amb la resta de la pgina
	}
	
	// Setters
	//
	public void setAction(String action) {
		this.action = action;
	}
	public void setValidateForm(boolean validateForm) {
		this.validateForm = validateForm;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	public void setTokenCheck(boolean tokenCheck) {
		this.tokenCheck = tokenCheck;
	}
	public void setTabindex(Integer tabindex) {
		this.tabindex = tabindex;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	public void addParameter(String key, String value) throws JspException {
		if (key != null && value != null) {
			if (PatternUtil.isJewelExpression(value)) {
				value = String.valueOf(ParameterUtil.getExpressionValue(value, pageContext));
			}
			this.parametres.put(key, value);
		}
	}
	
}
