/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import org.jewel.exceptions.InvalidPathPatternException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.UrlUtil;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:commandLink&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class CommandLinkTag extends AbstractHtmlTag {

	private static final long serialVersionUID = -9014328704168137751L;

	// Plantilles de les etiquetes HTML
	private static final String BUTTON_TPL = "<a href=\"%s\" %s>%s</a>";
	
	// Atributs de l'etiqueta
	protected String action;
	protected String label;
	// Atributs propis de l'etiqueta HTML <a>
	protected String target;
	protected String rel;
	protected Integer tabindex;
	
	//
	private Map<String, String> parametres;
	
	@Override
	public int doStartTag() throws JspException {
		this.parametres = new HashMap<String, String>();
		
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			String value = (label != null ?  (String) ParameterUtil.getExpressionValue(label, pageContext) : "Submit");
			
			String completeUrl = UrlUtil.getUrl(((HttpServletRequest) pageContext.getRequest()).getContextPath(), action, parametres);
			
			StringBuffer attributes = getParentAttributes();
			
			if (target != null)
				attributes.append(" target=\"" + target + "\" ");
			if (rel != null)
				attributes.append(" rel=\"" + rel + "\" ");
			if (tabindex != null)
				attributes.append(" tabindex=\"" + tabindex + "\" ");
			
			pageContext.getOut().println(String.format(BUTTON_TPL, completeUrl, attributes.toString(), value));
		} catch (IOException e) {
			throw new JspException("Error: IOException", e);
		} catch (InvalidPathPatternException e) {
			throw new JspException("Patr URL invalid al bot " + id + "\n" + e.getMessage());
		}
		
		return EVAL_PAGE; // Seguir amb la resta de la pgina
	}
	
	// Setters
	//
	public void setAction(String action) {
		this.action = action;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	public void setTarget(String target) {
		this.target = target;
	}
	public void setRel(String rel) {
		this.rel = rel;
	}
	public void setTabindex(Integer tabindex) {
		this.tabindex = tabindex;
	}
	public void addParameter(String key, String value) {
		if (key != null && value != null) {
			this.parametres.put(key, value);
		}
	}
	
}
