/**
 * 
 */
package org.jewel.taglib;

import java.util.ResourceBundle;

import javax.servlet.jsp.PageContext;

import org.jewel.core.ApplicationConfiguration;
import org.jewel.exceptions.InvalidContentException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;

/**
 * <p>Classe utilitzada per enmagatzemar les propietats dels elements d'un dataTable.</p>
 * <p>S'utilitza a l'etiqueta {@link ColumnTag} per transferir les dades dels elements 
 * de la taula a l'etiqueta {@link DataTableTag}</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class DataTableColumn implements DataTableColumnInterface {

	private String valueName;
	private String title;
	private String headerStyle;
	private String headerStyleClass;
	private String cellStyle;
	private String cellStyleClass;
	private String attributes;
	
	
	@Override
	public String getContent(String variableName, Object item, PageContext pageContext) throws InvalidContentException {
		String value = null;
		if (PatternUtil.isJewelExpression(valueName)) {
			String propertyName = PatternUtil.getExpression(valueName).replace(variableName+".", "");
			try {
				Object aux = ParameterUtil.getProperty(item, propertyName);
				if (aux == null) value = ""; // No volem que aparegui el valor 'null'
				else value = String.valueOf(aux);
			} catch (NoSuchMethodException e) {
				throw new InvalidContentException(e.getMessage());
			}
		}
		else if (PatternUtil.isJewelMessageExpression(valueName)) {
			ApplicationConfiguration config = ApplicationConfiguration.getInstance();
			ResourceBundle rb = (ResourceBundle) pageContext.getRequest().getAttribute(config.getLocaleConfig().getMessagesVariable());
			
			value = PatternUtil.getMessageExpression(valueName, rb);
		}
		else {
			value = valueName;
		}
		return value;
	}
	
	public String getValueName() {
		return valueName;
	}
	public void setValueName(String valueName) {
		this.valueName = valueName;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public String getHeaderStyle() {
		return headerStyle;
	}
	public void setHeaderStyle(String headerStyle) {
		this.headerStyle = headerStyle;
	}
	public String getHeaderStyleClass() {
		return headerStyleClass;
	}
	public void setHeaderStyleClass(String headerStyleClass) {
		this.headerStyleClass = headerStyleClass;
	}
	public String getCellStyle() {
		return cellStyle;
	}
	public void setCellStyle(String cellStyle) {
		this.cellStyle = cellStyle;
	}
	public String getCellStyleClass() {
		return cellStyleClass;
	}
	public void setCellStyleClass(String cellStyleClass) {
		this.cellStyleClass = cellStyleClass;
	}
	public String getAttributes() {
		return attributes;
	}
	public void setAttributes(String attributes) {
		this.attributes = attributes;
	}
	
}
