/**
 * 
 */
package org.jewel.taglib;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Map.Entry;

import javax.servlet.jsp.PageContext;

import org.jewel.core.ApplicationConfiguration;
import org.jewel.exceptions.InvalidContentException;
import org.jewel.exceptions.InvalidPathPatternException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;
import org.jewel.util.UrlUtil;

/**
 * <p>Classe utilitzada per enmagatzemar les propietats de les columnes d'acci d'un dataTable.</p>
 * <p>S'utilitza a l'etiqueta {@link ColumnActionTag} per transferir a l'etiqueta {@link DataTableTag}
 * les dades de les columnes amb botons o enllaos d'accions d'una taula</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class DataTableColumnAction implements DataTableColumnInterface {

	private static final String ACTION_IMG_TPL = "<img src=\"%s\" onclick=\"location.href='%s';\" title=\"%s\" />";
	private static final String ACTION_LINK_TPL = "<a href=\"%s\">%s</a>";
	
	private String baseUrl;
	private String action;
	private String actionImg;
	private String actionText;
	private Map<String, String> actionParametres;
	private String title;
	private String headerStyle;
	private String headerStyleClass;
	private String cellStyle;
	private String cellStyleClass;
	private String attributes;
	
	
	@Override
	public String getContent(String variableName, Object item, PageContext pageContext) throws InvalidContentException {
		String completeUrl = "";
		try {
			Map<String, String> valuedParameters = new HashMap<String, String>();
			for (Entry<String, String> param : actionParametres.entrySet()) {
				String value;
				if (PatternUtil.isJewelExpression(param.getValue())) {
					String valueName = PatternUtil.getExpression(param.getValue()).replace(variableName+".", "");
					try {
						value = String.valueOf(ParameterUtil.getProperty(item, valueName));
					} catch (NoSuchMethodException e) {
						throw new InvalidContentException(e.getMessage());
					}
				}
				else {
					value = param.getValue();
				}
				valuedParameters.put(param.getKey(), value);
			}
			
			completeUrl = UrlUtil.getUrl(baseUrl, action, valuedParameters);
		} catch (InvalidPathPatternException e) {
			throw new InvalidContentException(e.getMessage());
		}
		
		String sText = "";
		if (PatternUtil.isJewelExpression(actionText)) {
			String propertyName = PatternUtil.getExpression(actionText).replace(variableName+".", "");
			try {
				Object aux = ParameterUtil.getProperty(item, propertyName);
				if (aux == null) sText = ""; // No volem que aparegui el valor 'null'
				else sText = String.valueOf(aux);
			} catch (NoSuchMethodException e) {
				throw new InvalidContentException(e.getMessage());
			}
		}
		else if (PatternUtil.isJewelMessageExpression(actionText)) {
			ApplicationConfiguration config = ApplicationConfiguration.getInstance();
			ResourceBundle rb = (ResourceBundle) pageContext.getRequest().getAttribute(config.getLocaleConfig().getMessagesVariable());
			
			sText = PatternUtil.getMessageExpression(actionText, rb);
		}
		else {
			sText = actionText;
		}
		
		String content = "<<" + action + ">>";
		if (actionImg != null) {
			content = String.format(ACTION_IMG_TPL, actionImg, completeUrl, sText);
		}
		else if (actionText != null) {
			content = String.format(ACTION_LINK_TPL, completeUrl, sText);
		}
		return content;
	}
	
	
	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}
	public void setAction(String action) {
		this.action = action;
	}
	public void setActionImg(String actionImg) {
		this.actionImg = actionImg;
	}
	public void setActionText(String actionText) {
		this.actionText = actionText;
	}
	public void setActionParametres(Map<String, String> actionParametres) {
		this.actionParametres = actionParametres;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public String getHeaderStyle() {
		return headerStyle;
	}
	public void setHeaderStyle(String headerStyle) {
		this.headerStyle = headerStyle;
	}
	public String getHeaderStyleClass() {
		return headerStyleClass;
	}
	public void setHeaderStyleClass(String headerStyleClass) {
		this.headerStyleClass = headerStyleClass;
	}
	public String getCellStyle() {
		return cellStyle;
	}
	public void setCellStyle(String cellStyle) {
		this.cellStyle = cellStyle;
	}
	public String getCellStyleClass() {
		return cellStyleClass;
	}
	public void setCellStyleClass(String cellStyleClass) {
		this.cellStyleClass = cellStyleClass;
	}
	public String getAttributes() {
		return attributes;
	}
	public void setAttributes(String attributes) {
		this.attributes = attributes;
	}
}
