/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.jewel.exceptions.InvalidContentException;
import org.jewel.util.ParameterUtil;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:dataTable&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class DataTableTag extends AbstractHtmlTag {

	private static final long serialVersionUID = -3498010886408207451L;
	
	private static final String TABLE_INIT_TPL = "<table %s>";
	private static final String ROW_TPL = "<tr class=\"%s\">%s</tr>";
	private static final String CELL_HEADER_TPL = "<th style=\"%s\" class=\"%s\">%s</th>";
	private static final String CELL_TPL = "<td style=\"%s\" class=\"%s\">%s</td>";
	private static final String TABLE_FIN_TPL = "</table>";
	
	// Atributs de l'etiqueta
	private Object elements;
	private String var;
	private String rowStyleClasses;
	private String emptyMessage;
	// Atributs HTML de la taula
	private String border;
	private String cellpadding;
	private String cellspacing;
	private String width;
	
	// Camps de la classe
	@SuppressWarnings("rawtypes")
	Collection llistat;
	private List<DataTableColumnInterface> columns;
	
	
	@SuppressWarnings("rawtypes")
	@Override
	public int doStartTag() throws JspException {
		Object bean;
		if (elements instanceof String) {
			bean = ParameterUtil.getExpressionValue((String) elements, pageContext);
		}
		else {
			bean = elements;
		}
		if (bean != null) {
			try {
				llistat = (Collection) bean;
				
				// Inicialitzam les variables columnNames i columnHeaders per a que es pugui
				// comenar a afegir noms
				columns = new ArrayList<DataTableColumnInterface>();
			}
			catch (ClassCastException e) {
				throw new JspException("L'atribut 'elements' del dataTable '" + getId() + "' no s una collecci vlida.", e);
			}
		}
		
		return EVAL_BODY_INCLUDE;
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			StringBuffer attributes = getParentAttributes();
			if (border != null)
				attributes.append(" border=\"" + border + "\"");
			if (cellpadding != null)
				attributes.append(" cellpadding=\"" + cellpadding + "\"");
			if (cellspacing != null)
				attributes.append(" cellspacing=\"" + cellspacing + "\"");
			if (width != null)
				attributes.append(" width=\"" + width + "\"");
			pageContext.getOut().println(String.format(TABLE_INIT_TPL, attributes.toString()));
			
			
			String[] rowClass = {""};
			if (rowStyleClasses != null && !"".equals(rowStyleClasses)) rowClass = rowStyleClasses.split(",");
			int rowCount = 0;
			
			StringBuffer headerContent = new StringBuffer();
			// Posam les capaleres
			for (DataTableColumnInterface column : columns) {
				headerContent.append(
						String.format(CELL_HEADER_TPL, column.getHeaderStyle(), column.getHeaderStyleClass(), column.getTitle()));
			}
			pageContext.getOut().println(String.format(ROW_TPL, rowClass[rowCount++ % rowClass.length], headerContent.toString()));
			
			// Posam totes les files
			if (llistat != null && llistat.size() > 0) {
				for (Object item : llistat) {
					StringBuffer rowContent = new StringBuffer();
					for (DataTableColumnInterface column : columns) {
						rowContent.append(
								String.format(CELL_TPL, column.getCellStyle(), column.getCellStyleClass(), column.getContent(var, item, pageContext)));
					}
					pageContext.getOut().println(String.format(ROW_TPL, rowClass[rowCount++ % rowClass.length], rowContent.toString()));
				}
			}
			else if (emptyMessage != null) {
				pageContext.getOut().println(String.format(ROW_TPL, rowClass[rowCount % rowClass.length], 
						"<td colspan=\"" + columns.size() + "\">" + ParameterUtil.getExpressionValue(emptyMessage, pageContext) + "</td>"));
			}
			
			pageContext.getOut().println(TABLE_FIN_TPL);
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		} catch (InvalidContentException e) {
			throw new JspException("Error: InvalidContentException ", e);
		}
		
		return super.doEndTag();
	}
	

	public void addColumn(DataTableColumnInterface column) {
		columns.add(column);
	}
	
	// Setters
	//
	public void setElements(Object elements) {
		this.elements = elements;
	}
	public void setVar(String var) {
		this.var = var;
	}
	public void setRowStyleClasses(String rowStyleClasses) {
		this.rowStyleClasses = rowStyleClasses;
	}
	public void setEmptyMessage(String emptyMessage) {
		this.emptyMessage = emptyMessage;
	}
	public void setBorder(String border) {
		this.border = border;
	}
	public void setCellpadding(String cellpadding) {
		this.cellpadding = cellpadding;
	}
	public void setCellspacing(String cellspacing) {
		this.cellspacing = cellspacing;
	}
	public void setWidth(String width) {
		this.width = width;
	}
	
}
