/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.jewel.util.JewelConstants;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:form&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class FormTag extends AbstractHtmlTag {

	private static final long serialVersionUID = -4143942487572863074L;
	
	// Plantilles de les etiquetes HTML
	private static final String FORM_INI_TPL = "<form method=\"%s\" name=\"%s\" %s>";
	private static final String FORM_FIN_TPL = "</form>";
	private static final String TOKEN_INPUT_TPL = "<input type=\"hidden\" name=\"%s\" value=\"%s\" />";
	
	// Atributs de l'etiqueta
	
	// Atributs propis de l'etiqueta HTML <a>
	protected String method;
	protected String target;
	private String name;
	
	//
	private List<String> validations;
	
	
	@Override
	public int doStartTag() throws JspException {
		if (pageContext.getAttribute("current_form") != null) throw new JspException("Cannot put form tag into another form tag.");
		if (name == null) throw new JspException("Un formulari necessita l'atribut 'name'.");
		
		pageContext.setAttribute("current_form", name);
		
		validations = new ArrayList<String>();
		try {
			StringBuffer attributes = getParentAttributes();
			
			if (target != null)
				attributes.append(" target=\"" + target + "\"");
			
			pageContext.getOut().println(String.format(FORM_INI_TPL, (method != null ? method : "POST"), name, attributes.toString()));
			
			// Afegim el camp que ens servir per validar el token de petici
			//
			pageContext.getOut().println(String.format(TOKEN_INPUT_TPL, 
					JewelConstants.CURRENT_TOKEN_KEY, pageContext.getSession().getAttribute(JewelConstants.CURRENT_TOKEN_KEY)));
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			StringBuffer sbValidation = new StringBuffer("<script type=\"text/javascript\">");
			sbValidation.append("function validateForm" + name + "() {");
			for (String sValidation : validations) {
				sbValidation.append(sValidation);
			}
			sbValidation.append("return true;");
			sbValidation.append("}");
			sbValidation.append("</script>");
			
			pageContext.getOut().println(sbValidation.toString());
			pageContext.getOut().println(FORM_FIN_TPL);
			pageContext.removeAttribute("current_form");
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_PAGE; // Seguir amb la resta de la pgina
	}
	
	
	/**
	 * <p>Afegeix una validaci 'required' per un camp del formulari</p>
	 * 
	 * @param nameField Name del camp a validar.
	 * @param errorMessage Missatge que es mostrar en cas de no validar-se
	 * Si <i>errorMessage</i> s null, es mostrar "El camp <i>nameField</i> s obligatori."
	 */
	public void addRequiredValidation(String nameField, String errorMessage) {
		if (nameField == null || "".equals(nameField)) return;
		
		StringBuffer sValidation = new StringBuffer();
		sValidation.append("if (document.forms[\"" + name + "\"][\"" + nameField + "\"].value == null ");
		sValidation.append(			" || document.forms[\"" + name + "\"][\"" + nameField + "\"].value == \"\") {");
		if (errorMessage != null && !"".equals(errorMessage)) {
			sValidation.append("alert(\"" + errorMessage + "\");");
		} else {
			sValidation.append("alert(\"El camp '"+ nameField +"' s obligatori.\");");
		}
		sValidation.append(		"return false;");
		sValidation.append("}");
		
		validations.add(sValidation.toString());
	}
	
	
	/**
	 * Afegeix una validaci al formulari
	 * 
	 * @param validationScript String que cont el Javascript que s'executar al validar el formulari.
	 */
	public void addValidation(String validationScript) {
		validations.add(validationScript);
	}
	
	
	// Setters
	public void setMethod(String method) {
		this.method = method;
	}
	public void setTarget(String target) {
		this.target = target;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	// Getters
	public String getName() {
		return name;
	}
	
}
