/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;

import javax.servlet.jsp.JspException;

/**
 * <p>Classe que correspon a l'etiqueta &lt;jwl:inputDate&gt;</p>
 * <p>Al renderitzar l'INPUT s'utilitza <b>jQuery</b> per generar el datepicker.</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class InputDateTag extends AbstractInputTag {
	
	private static final long serialVersionUID = -4143942487572863074L;

	// Plantilles de les etiquetes HTML
	private static final String INPUT_TEXT_TPL = "<input type=\"text\" %s></input>"; // Camp que veur l'usuari
	private static final String INPUT_HIDDEN_TPL = "<input type=\"hidden\" value=\"%s\" %s></input>"; // Parametre real enviat
	
	// Atributs de l'etiqueta
	private String pattern;
	private String buttonImage;
	
	
	@Override
	public int doStartTag() throws JspException {
		if (pageContext.getAttribute("current_form") == null) throw new JspException("No pots posar un camp de data sense un formulari!");
		if (id == null) throw new JspException("El camp de data necessita un ID."); 
		
		if (pattern == null) setPattern("dd/mm/yy");

		if (isRequired() && getParent() instanceof FormTag) {
			if (name == null) throw new JspException("No pots posar un camp requerit sense l'atribut 'name'."); 
			FormTag formTag = (FormTag) getParent();
			formTag.addRequiredValidation(name, getRequiredMessage());
		}
		
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			// Camp INPUT on es veur la data del datepicker
			//
			StringBuffer attributes = new StringBuffer();
			// Posem els atributs sense cridar a getParentAttributes() ja que l'ID i el NAME
			// han de ser adaptats
			attributes.append(" id=\"datepicker_" + id + "\" ");
			if (name != null && !"".equals(name))
				attributes.append(" name=\"datepicker_" + name + "\" ");
			if (style != null && !"".equals(style))
				attributes.append(" style=\"" + style + "\" ");
			if (styleClass != null && !"".equals(styleClass))
				attributes.append(" class=\"" + styleClass + "\" ");
			if (disabled)
				attributes.append(" disabled=\"disabled\" ");
			if (size != null && size > 0)
				attributes.append(" size=\"" + size + "\"");
			if (tabindex != null && tabindex > 0)
				attributes.append(" tabindex=\"" + tabindex + "\"");
			// Nomes es pot seleccionar des del calendari datepicker
			attributes.append(" readonly=\"readonly\"");
			pageContext.getOut().println(String.format(INPUT_TEXT_TPL, attributes.toString()));
			
			Object dateValue = getValue();
			
			// Camp hidden amb el valor real de la data (en milisegons)
			//
			StringBuffer hiddenAttributes = new StringBuffer();
			hiddenAttributes.append(" id=\"" + id + "\" ");
			if (name != null && !"".equals(name))
				hiddenAttributes.append(" name=\"" + name + "\" ");
			if (disabled)
				hiddenAttributes.append(" disabled=\"disabled\" ");
			pageContext.getOut().println(String.format(INPUT_HIDDEN_TPL, dateValue, hiddenAttributes.toString()));
			
			// Javascript utilitzat per omplir el camp HIDDEN amb la data en milisegons i el valor dels dos camps
			//
			pageContext.getOut().println("<script type=\"text/javascript\">");
			pageContext.getOut().println("if (typeof jQuery == 'undefined') {"); // No t jQuery
			pageContext.getOut().println("alert('No es poden utilitzar camps datepicker sense jQuery');");
			pageContext.getOut().println("} else {"); // No t jQuery
			if (dateValue != null && !"".equals(dateValue)) {
				pageContext.getOut().println(" var dataDefecte = $.datepicker.parseDate('@', '" + (Long) dateValue + "');");
				pageContext.getOut().println(" var sDataDefecte = $.datepicker.formatDate('" + this.pattern +"', dataDefecte);");
				pageContext.getOut().println("$('#datepicker_" + this.id + "').val(sDataDefecte);");
			}
			StringBuffer datePickerAttr = new StringBuffer(" dateFormat: '" + this.pattern + "' ");
			if (this.buttonImage != null) {
				datePickerAttr.append(" , showOn: \"button\", buttonImage: \"" + this.buttonImage + "\", buttonImageOnly: true ");
			}
			datePickerAttr.append(" , onSelect: function(dateText, inst) { ");
			datePickerAttr.append(		"var dataLong = $.datepicker.formatDate(\"@\", $(this).datepicker('getDate'));");
			datePickerAttr.append(		"$('#" + id + "').val(dataLong);");
			datePickerAttr.append("} ");
			
			pageContext.getOut().println("$(function() { $('#datepicker_" + this.id + "').datepicker({ " + datePickerAttr.toString() + " }); });");
			pageContext.getOut().println("}");
			pageContext.getOut().println("</script>");
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_PAGE; // Seguir amb la resta de la pgina
	}

	
	// Setters
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}
	public void setButtonImage(String buttonImage) {
		this.buttonImage = buttonImage;
	}
	
}
