/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.jewel.exceptions.InvalidContentException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;

/**
 * <p>Classe que correspon a l'etiqueta &lt;jwl:option&gt;</p>
 * <p>Pot generar opcions per les etiquetes &lt;jwl:select&gt; {@link SelectTag}, &lt;jwl:selectCheckbox&gt;
 * {@link SelectCheckboxTag} i &lt;jwl:selectRadio&gt; {@link SelectRadioTag}</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class OptionTag extends AbstractHtmlTag {

	private static final long serialVersionUID = 4993872374509548938L;

	// Etiquetes HTML
	private static final String SELECT_OPTION_TPL = "<option value=\"%s\" %s>%s</option>";
	private static final String RADIO_OPTION_TPL = "<input type=\"radio\" name=\"%s\" value=\"%s\" %s>%s</input>";
	private static final String CHECK_OPTION_TPL = "<input type=\"checkbox\" name=\"%s\" value=\"%s\" %s>%s</option>";
	
	// Atributs de l'etiqueta
	protected String value;
	protected String label;
	// Atributs propis de l'etiqueta HTML <option>
	protected boolean selected;
	protected boolean disabled;
	
	@SuppressWarnings("unchecked")
	@Override
	public int doStartTag() throws JspException {
		if (!(getParent() instanceof SelectTag) && !(getParent() instanceof SelectRadioTag) && !(getParent() instanceof SelectCheckboxTag))
			throw new JspException("L'etiqueta d'opci ha d'estar dins una etiqueta SelectTag, SelectRadioTag o SelectCheckboxTag");
		
		try {
			StringBuffer attributes = getParentAttributes();
			if (disabled)
				attributes.append(" disabled=\"disabled\" ");
			
			if (getParent() instanceof SelectTag) {
				if (selected)
					attributes.append(" selected=\"selected\" ");
				
				pageContext.getOut().append(String.format(SELECT_OPTION_TPL, 
						ParameterUtil.getExpressionValue(this.value, pageContext), attributes.toString(), ParameterUtil.getExpressionValue(this.label, pageContext)));
			}
			else if (getParent() instanceof SelectRadioTag) {
				SelectRadioTag selectTag = (SelectRadioTag) getParent();
				if (selected)
					attributes.append(" checked=\"checked\" ");
				if (selectTag.getSize() != null && selectTag.getSize() > 0)
					attributes.append(" size=\"" + selectTag.getSize() + "\"");
				if (selectTag.isReadonly())
					attributes.append(" readonly=\"readonly\" ");
				
				pageContext.getOut().append(String.format(RADIO_OPTION_TPL, 
						selectTag.getName(), ParameterUtil.getExpressionValue(this.value, pageContext), attributes.toString(), ParameterUtil.getExpressionValue(this.label, pageContext)));
			}
			else if (getParent() instanceof SelectCheckboxTag) {
				SelectCheckboxTag selectTag = (SelectCheckboxTag) getParent();
				Collection<Object> selectValue = null;
				Object selectValueAux = selectTag.getValue();
				if (selectValueAux instanceof Collection) {
					selectValue = (Collection<Object>) selectValueAux;
				}
				else {
					selectValue = new ArrayList<Object>();
					selectValue.add(selectValueAux);
				}
				// Obtenim els camps per el qual es comparar el valor
				List<Object> selectValues = new ArrayList<Object>();
				for (Object o : selectValue) {
					Object valor = getExpressionOption(o, this.value);
					selectValues.add(valor);
				}
				
				// Afegim els atributs de l'etiqueta
				Object itemValue = ParameterUtil.getExpressionValue(this.value, pageContext);
				if (selected || selectValue != null && selectValues.contains(itemValue))
					attributes.append(" checked=\"checked\" ");
				if (selectTag.getSize() != null && selectTag.getSize() > 0)
					attributes.append(" size=\"" + selectTag.getSize() + "\"");
				if (selectTag.isReadonly())
					attributes.append(" readonly=\"readonly\" ");
				
				// Creem l'etiqueta HTML
				pageContext.getOut().append(String.format(CHECK_OPTION_TPL, selectTag.getName(), itemValue, attributes.toString(), 
						ParameterUtil.getExpressionValue(this.label, pageContext)));
			
			}
		}
		catch (IOException e) {
			throw new JspException("IOException: ", e);
		}
		catch (InvalidContentException e) {
			throw new JspException("L'etiqueta options '" + getId() + "' t el camp value o el camp label invlid.", e);
		}
		
		
		return super.doStartTag();
	}
	
	
	private Object getExpressionOption(Object element, String valueName) throws InvalidContentException, JspException {
		Object value = null;
		if (PatternUtil.isJewelExpression(valueName)) {
			if (valueName.contains(".")) {
				try {
					value = ParameterUtil.getProperty(element, valueName);
				} catch (NoSuchMethodException e) {
					throw new InvalidContentException(e.getMessage());
				}
			}
			else {
				value = element;
			}
		}
		else {
			value = valueName;
		}
		return value;
	}
	
	
	// Setters
	public void setValue(String value) {
		this.value = value;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	public void setSelected(boolean selected) {
		this.selected = selected;
	}
	public void setDisabled(boolean disabled) {
		this.disabled = disabled;
	}
	
}
