/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.jewel.exceptions.InvalidContentException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;

/**
 * <p>Classe que correspon a l'etiqueta &lt;jwl:options&gt;</p>
 * <p>Pot generar opcions per les etiquetes &lt;jwl:select&gt; {@link SelectTag}, &lt;jwl:selectCheckbox&gt;
 * {@link SelectCheckboxTag} i &lt;jwl:selectRadio&gt; {@link SelectRadioTag}</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class OptionsTag extends AbstractHtmlTag {

	private static final long serialVersionUID = 4993872374509548938L;

	// Etiquetes HTML
	private static final String SELECT_OPTION_TPL = "<option value=\"%s\" %s %s>%s</option>";
	private static final String RADIO_OPTION_TPL = "<input type=\"radio\" name=\"%s\" value=\"%s\" %s %s>%s</option>";
	private static final String CHECK_OPTION_TPL = "<input type=\"checkbox\" name=\"%s\" value=\"%s\" %s %s>%s</option>";
	
	// Atributs de l'etiqueta
	private Object elements;
	private String var;
	protected String value;
	protected String label;
	
	@SuppressWarnings("unchecked")
	@Override
	public int doStartTag() throws JspException {
		if (!(getParent() instanceof SelectTag) && !(getParent() instanceof SelectRadioTag) && !(getParent() instanceof SelectCheckboxTag))
			throw new JspException("L'etiqueta d'opci ha d'estar dins una etiqueta SelectTag, SelectRadioTag o SelectCheckboxTag");
		
		try {
			Object bean;
			if (elements instanceof String) {
				bean = ParameterUtil.getExpressionValue((String) elements, pageContext);
			}
			else {
				bean = elements;
			}
			if (bean != null && !(bean instanceof String)) {
				if (!(bean instanceof Collection)) {
					throw new JspException("L'atribut 'elements' de l'etiqueta options '" + getId() + "' no s una collecci vlida.");
				}
				try {
					@SuppressWarnings("rawtypes")
					Collection llistat = (Collection) bean;
					if (llistat != null && llistat.size() > 0) {
						StringBuffer attributes = getParentAttributes();
						
						if (getParent() instanceof SelectTag) {
							SelectTag selectTag = (SelectTag) getParent();
							Object selectValue = selectTag.getValue();
							for (Object element : llistat) {
								Object itemValue = getExpressionOption(element, this.value);
								String sSelected = (selectValue != null && selectValue.equals(itemValue) ? " selected=\"selected\" " : "");
								pageContext.getOut().append(String.format(SELECT_OPTION_TPL, itemValue, sSelected, attributes.toString(), getExpressionOption(element, this.label)));
							}
						}
						else if (getParent() instanceof SelectRadioTag) {
							SelectRadioTag selectTag = (SelectRadioTag) getParent();
							Object selectValue = selectTag.getValue();
							if (selectTag.getSize() != null && selectTag.getSize() > 0)
								attributes.append(" size=\"" + selectTag.getSize() + "\"");
							if (selectTag.isDisabled())
								attributes.append(" disabled=\"disabled\" ");
							if (selectTag.isReadonly())
								attributes.append(" readonly=\"readonly\" ");
							for (Object element : llistat) {
								Object itemValue = getExpressionOption(element, this.value);
								String sSelected = (selectValue != null && selectValue.equals(itemValue) ? " checked=\"checked\" " : "");
								pageContext.getOut().append(String.format(RADIO_OPTION_TPL, selectTag.getName(), itemValue, sSelected, attributes.toString(), getExpressionOption(element, this.label)));
							}
						}
						else if (getParent() instanceof SelectCheckboxTag) {
							SelectCheckboxTag selectTag = (SelectCheckboxTag) getParent();
							Collection<Object> selectValue = null;
							Object selectValueAux = selectTag.getValue();
							if (selectValueAux instanceof Collection) {
								selectValue = (Collection<Object>) selectValueAux;
							}
							else {
								selectValue = new ArrayList<Object>();
								if (!"".equals(selectValueAux)) {
									selectValue.add(selectValueAux);
								}
							}
							// Obtenim els camps per el qual es comparar el valor
							List<Object> selectValues = new ArrayList<Object>();
							for (Object o : selectValue) {
								Object valor = getExpressionOption(o, this.value);
								selectValues.add(valor);
							}
							
							// Afegim els atributs de l'etiqueta
							if (selectTag.getSize() != null && selectTag.getSize() > 0)
								attributes.append(" size=\"" + selectTag.getSize() + "\"");
							if (selectTag.isDisabled())
								attributes.append(" disabled=\"disabled\" ");
							if (selectTag.isReadonly())
								attributes.append(" readonly=\"readonly\" ");
							
							// Recorrem les opcions i creem les etiquetes HTML
							for (Object element : llistat) {
								Object itemValue = getExpressionOption(element, this.value);
								String sSelected = (selectValue != null && selectValues.contains(itemValue) ? " checked=\"checked\" " : "");
								pageContext.getOut().append(String.format(CHECK_OPTION_TPL, selectTag.getName(), itemValue, sSelected, attributes.toString(), getExpressionOption(element, this.label)));
							}
						}
					}
				}
				catch (InvalidContentException e) {
					throw new JspException("L'etiqueta options '" + getId() + "' t el camp value o el camp label invlid.", e);
				}
			}
		}
		catch (IOException e) {
			throw new JspException("IOException: ", e);
		}
		
		
		return super.doStartTag();
	}
	
	private Object getExpressionOption(Object element, String valueName) throws InvalidContentException, JspException {
		Object value = null;
		if (PatternUtil.isJewelExpression(valueName)) {
			if (valueName.contains(".")) {
				String propertyName = PatternUtil.getExpression(valueName).replace(this.var+".", "");
				try {
					value = ParameterUtil.getProperty(element, propertyName);
				} catch (NoSuchMethodException e) {
					throw new InvalidContentException(e.getMessage());
				}
			}
			else {
				value = element;
			}
		}
		else {
			value = valueName;
		}
		return value;
	}

	// Setters
	public void setElements(Object elements) {
		this.elements = elements;
	}
	public void setVar(String var) {
		this.var = var;
	}
	public void setValue(String value) {
		this.value = value;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	
}
