/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.Date;

import javax.servlet.jsp.JspException;

import org.jewel.util.DateUtil;
import org.jewel.util.ParameterUtil;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:outputText&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class OutputDateTag extends AbstractHtmlTag {
	
	private static final long serialVersionUID = -4143942487572863074L;

	// Plantilles de les etiquetes HTML
	private static final String TEXT_TPL = "<span %s>%s</span>";
	
	// Atributs de l'etiqueta
	private Object date;
	protected String pattern;
	protected boolean onlyText;
	
	@Override
	public int doStartTag() throws JspException {
		try {
			Date ddate = null;
			String fData;
			
			if (date instanceof Date) {
				ddate = (Date) date;
			}
			else {
				Object dateAux = ParameterUtil.getExpressionValue((String) this.date, pageContext);
				if (dateAux != null && dateAux instanceof Long) {
					ddate = new Date((Long) dateAux);
				}
			}
			fData = (ddate != null ? DateUtil.formatDate(ddate, pattern) : "");
			
			if (onlyText) {
				pageContext.getOut().print(fData);
			}
			else {
				// Afegim el text dins un SPAN
				pageContext.getOut().print(String.format(TEXT_TPL, 
						getParentAttributes().toString(), fData));
			}
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		catch (NumberFormatException e) {
			throw new JspException("Error al convertir '" + date + "' a tipus java.util.Date");
		}
		catch (ClassCastException e) {
			throw new JspException("Error al convertir '" + date + "' a tipus java.util.Date");
		}
		
		return EVAL_PAGE;
	}
	
	
	// Setters
	public void setDate(Object date) {
		this.date = date;
	}
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}
	public void setOnlyText(boolean onlyText) {
		this.onlyText = onlyText;
	}
	
}
