/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.jewel.util.ParameterUtil;

/**
 * <p>Classe que correspon a l'etiqueta &lt;jwl:selectRadio&gt;</p>
 * <p>Aquesta etiqueta pot contenir etiquetes &lt;jwl:options&gt; {@link OptionsTag}
 * i etiquetes &lt;jwl:option&gt; {@link OptionTag} que representen les opcions del grup</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class SelectRadioTag extends AbstractInputTag {

	private static final long serialVersionUID = 721589398032716945L;
	
	// Plantilles de les etiquetes HTML
	private static final String RADIO_INIT_TPL = "<div %s>";
	private static final String RADIO_END_TPL = "</div>";
	private static final String RADIO_EMPTY_OPTION = "<input type=\"radio\" name=\"%s\" value=\"%s\">%s</option>";
	
	// Atributs de l'etiqueta
	protected boolean emptyOption;
	protected String emptyOptionValue;
	protected String emptyOptionLabel;
	
	
	@Override
	public int doStartTag() throws JspException {
		try {
			StringBuffer attributes = getParentAttributes();
		
			if (isRequired() && getParent() instanceof FormTag) {
				if (name == null) throw new JspException("No pots posar un camp requerit sense l'atribut 'name'."); 
				FormTag formTag = (FormTag) getParent();
				formTag.addValidation(getRadioValidation(formTag.getName()));
			}
			String sEmptyValue = "";
			if (this.emptyOptionValue != null) {
				sEmptyValue = (String) ParameterUtil.getExpressionValue(this.emptyOptionValue, pageContext);
			}
			String sEmptyLabel = "Select...";
			if (this.emptyOptionLabel != null) {
				sEmptyLabel = (String) ParameterUtil.getExpressionValue(this.emptyOptionLabel, pageContext);
			}
			pageContext.getOut().println(String.format(RADIO_INIT_TPL, attributes.toString()));
			if (emptyOption) {
				pageContext.getOut().println(String.format(RADIO_EMPTY_OPTION, name, sEmptyValue, sEmptyLabel));
			}
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_BODY_INCLUDE;
	}
	
	private String getRadioValidation(String formName) throws JspException {
		StringBuffer sValidation = new StringBuffer();
		sValidation.append("var val = 0;");
		sValidation.append("var chkGroup = document.forms[\"" + formName + "\"]." + this.name + ";");
		sValidation.append("for (var i=0; i<chkGroup.length; i ++) {");
		sValidation.append(	" if (chkGroup[i].checked == true) val = chkGroup[i].value; ");
		sValidation.append("}");
		sValidation.append("if (val == 0) {");
		if (requiredMessage != null && !"".equals(requiredMessage)) {
			sValidation.append("alert(\"" + getRequiredMessage() + "\");");
		} else {
			sValidation.append("alert(\"El camp '"+ this.name +"' s obligatori.\");");
		}
		sValidation.append(		"return false;");
		sValidation.append("}");
		return sValidation.toString();
	}

	@Override
	public int doEndTag() throws JspException {
		try {
			pageContext.getOut().println(RADIO_END_TPL);
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		return EVAL_PAGE;
	}
	
	
	// Setters dels atributs
	public void setEmptyOption(boolean emptyOption) {
		this.emptyOption = emptyOption;
	}
	public void setEmptyOptionLabel(String emptyOptionLabel) {
		this.emptyOptionLabel = emptyOptionLabel;
	}
	public void setEmptyOptionValue(String emptyOptionValue) {
		this.emptyOptionValue = emptyOptionValue;
	}
	
	// Getters
	public String getName() {
		return name;
	}
	
}
