/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.jewel.util.ParameterUtil;

/**
 * <p>Classe que correspon a l'etiqueta &lt;jwl:select&gt;</p>
 * <p>Aquesta etiqueta pot contenir etiquetes &lt;jwl:options&gt; {@link OptionsTag}
 * i etiquetes &lt;jwl:option&gt; {@link OptionTag} que representen les opcions del desplegable</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class SelectTag extends AbstractInputTag {

	private static final long serialVersionUID = 721589398032716945L;
	
	// Plantilles de les etiquetes HTML
	private static final String SELECT_INIT_TPL = "<select %s>";
	private static final String SELECT_END_TPL = "</select>";
	private static final String SELECT_EMPTY_OPTION = "<option value=\"%s\">%s</option>";
	
	// Atributs de l'etiqueta
	protected boolean emptyOption;
	protected String emptyOptionValue = "";
	protected String emptyOptionLabel;
	// Atributs propis de l'etiqueta HTML <select>
	protected boolean multiple;
	
	
	@Override
	public int doStartTag() throws JspException {
		try {
			StringBuffer attributes = getParentAttributes();
			
			if (isRequired() && getParent() instanceof FormTag) {
				if (name == null) throw new JspException("No pots posar un camp requerit sense l'atribut 'name'."); 
				FormTag formTag = (FormTag) getParent();
				formTag.addRequiredValidation(name, getRequiredMessage());
			}
			
			if (this.multiple) {
				attributes.append(" multiple=\"multiple\" ");
			}
			String sEmptyValue = "";
			if (this.emptyOptionValue != null) {
				sEmptyValue = (String) ParameterUtil.getExpressionValue(this.emptyOptionValue, pageContext);
			}
			String sEmptyLabel = "Select...";
			if (this.emptyOptionLabel != null) {
				sEmptyLabel = (String) ParameterUtil.getExpressionValue(this.emptyOptionLabel, pageContext);
			}
			pageContext.getOut().println(String.format(SELECT_INIT_TPL, attributes.toString()));
			if (emptyOption) {
				pageContext.getOut().println(String.format(SELECT_EMPTY_OPTION, sEmptyValue, sEmptyLabel));
			}
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_BODY_INCLUDE;
	}
	
	@Override
	public int doEndTag() throws JspException {
		try {
			pageContext.getOut().println(SELECT_END_TPL);
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		return EVAL_PAGE;
	}
	
	
	// Setters dels atributs
	public void setEmptyOption(boolean emptyOption) {
		this.emptyOption = emptyOption;
	}
	public void setEmptyOptionLabel(String emptyOptionLabel) {
		this.emptyOptionLabel = emptyOptionLabel;
	}
	public void setEmptyOptionValue(String emptyOptionValue) {
		this.emptyOptionValue = emptyOptionValue;
	}
	public void setMultiple(boolean multiple) {
		this.multiple = multiple;
	}
	
}
