/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;

import javax.servlet.jsp.JspException;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:textarea&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class TextareaTag extends AbstractInputTag {
	
	private static final long serialVersionUID = -4143942487572863074L;

	// Plantilles de les etiquetes HTML
	private static final String TEXTAREA_INIT_TPL = "<textarea %s>%s";
	private static final String TEXTAREA_END_TPL = "</textarea>";
	
	// Atributs propis de l'etiqueta HTML <a>
	protected Integer cols;
	protected Integer rows;
	
	@Override
	public int doStartTag() throws JspException {
		if (pageContext.getAttribute("current_form") == null) 
			throw new JspException("No pots posar un camp textarea sense un formulari!"); 
		
		try {
			StringBuffer attributes = getParentAttributes();
			
			if (isRequired() && getParent() instanceof FormTag) {
				if (name == null) throw new JspException("No pots posar un camp requerit sense l'atribut 'name'."); 
				FormTag formTag = (FormTag) getParent();
				formTag.addRequiredValidation(name, getRequiredMessage());
			}
			
			if (cols != null && cols > 0)
				attributes.append(" cols=\"" + cols + "\"");
			if (rows != null && rows > 0)
				attributes.append(" rows=\"" + rows + "\"");
			
			pageContext.getOut().println(String.format(TEXTAREA_INIT_TPL, attributes.toString(), getValue()));
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		try {
			pageContext.getOut().println(TEXTAREA_END_TPL);
		} catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		
		return EVAL_PAGE; // Seguir amb la resta de la pgina
	}
	
	
	// Setters
	public void setCols(Integer cols) {
		this.cols = cols;
	}
	public void setRows(Integer rows) {
		this.rows = rows;
	}
	
}
