/**
 * 
 */
package org.jewel.taglib;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.jewel.exceptions.InvalidPathPatternException;
import org.jewel.util.ParameterUtil;
import org.jewel.util.PatternUtil;
import org.jewel.util.UrlUtil;

/**
 * Classe que correspon a l'etiqueta &lt;jwl:url&gt;
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class UrlTag extends TagSupport {

	private static final long serialVersionUID = -3333373272953404533L;
	
	// Atributs de l'etiqueta
	protected String action;
	protected boolean ajax;
	
	private Map<String, String> parametres;
	
	
	@Override
	public int doStartTag() throws JspException {
		this.parametres = new HashMap<String, String>();
	
		return EVAL_BODY_INCLUDE; // Evaluar el codi entre l'etiqueta d'inici i la final
	}
	
	
	@Override
	public int doEndTag() throws JspException {
		String ctxPath = ((HttpServletRequest) pageContext.getRequest()).getContextPath();
		
		try {
			if (action == null || "".equals(action)) {
				pageContext.getOut().print(ctxPath);
			}
			else {
				String completeUrl = UrlUtil.getUrl(ctxPath, action, parametres);
				if (this.ajax) {
					completeUrl += "?ajax=true";
				}
				pageContext.getOut().print(completeUrl);
			}
		}
		catch (InvalidPathPatternException e) {
			throw new JspException("Patr URL incorrecte: " + action + "\n" + e.getMessage());
		}
		catch (IOException e) {
			throw new JspException("Error: IOException", e);
		}
		return super.doEndTag();
	}
	
	
	// Setters
	//
	public void setAction(String action) {
		this.action = action;
	}
	public void setAjax(boolean ajax) {
		this.ajax = ajax;
	}
	
	
	public void addParameter(String key, String value) throws JspException {
		if (key != null && value != null) {
			if (PatternUtil.isJewelExpression(value)) {
				value = String.valueOf(ParameterUtil.getExpressionValue(value, pageContext));
			}
			this.parametres.put(key, value);
		}
	}
	
}
