/**
 * 
 */
package org.jewel.util;

import org.jewel.core.LocaleFilter;
import org.jewel.core.ModelAndView;
import org.jewel.core.TokenFilter;

/**
 * <p>Constants utilitzades al framework que guarden, per exemple, el nom clau que se li donar
 * a un determinat atribut de sessi o petici</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class JewelConstants {

	/**
	 * <p>Nom clau que se li dona a l'atribut de sessi on es guarda la mostra o token</p>
	 * <p>S'utilitza al filtre {@link TokenFilter}, on es comprova el reenviament de formularis</p>
	 */
	public static final String CURRENT_TOKEN_KEY = "jewel.current_token";
	
	/**
	 * <p>Nom clau que se li dona a l'atribut de sessi on es guarda l'idioma actual</p>
	 * <p>S'utilitza al filtre {@link LocaleFilter} per establir el paquet de recursos
	 * adequat a l'idioma actual de la sessi</p>
	 */
	public static final String CURRENT_LOCALE_KEY = "jewel.request_locale";
	
	/**
	 * <p>Nom clau que se li dona a l'atribut de sessi on es guarden tots els controladors de sessi</p>
	 * <p>S'utilitza per enmagatzemar els controladors de sessi que s'utilitzen i es consulten
	 * posteriorment a l'aplicaci</p>
	 */
	public static final String SESSION_CONTROLLERS_KEY = "jewel.session_controllers";
	
	/**
	 * <p>Nom clau que se li dona a l'atribut de petici on es guarda l'excepci provocada a una petici</p>
	 * <p>S'utilitza per posteriorment consultar-la a la pgina d'error</p>
	 */
	public static final String EXCEPTION_ERROR_KEY = "error";
	
	/**
	 * <p>Nom clau que se li dona a l'atribut de petici on es guarden els missatges d'error 
	 * posats durant l'execuci d'una acci a un controlador de l'usuari</p>
	 */
	public static final String ERROR_MESSAGES_KEY = "jewel.error_messages";
	
	/**
	 * <p>Nom clau que se li dona a l'atribut de petici on es guarden els missatges d'informaci 
	 * posats durant l'execuci d'una acci a un controlador de l'usuari</p>
	 */
	public static final String INFO_MESSAGES_KEY = "jewel.info_messages";
	
	/**
	 * <p>Nom clau que se li dona al parmetre inicial que permet configurar la ruta de l'arxiu de 
	 * configuraci del framework Jewel</p>
	 * <p>Aquesta ruta es pot configurar als parmetres inicials de JewelServlet al web.xml de l'aplicaci</p>
	 */
	public static final String INIT_PARAM_CONFIG = "org.jewel.CONFIG_FILE";

	/**
	 * <p>Nom clau que se li dona a l'atribut on es guarda el model i la vista actual.</p>
	 * <p>S'utilitza al moment de presentar la vista. Si no s'ha trobat cap resultat {@link ModelAndView} al executar l'acci,
	 * s'ha de redirigir a la vista actual, que es guarda a un atribut de sessi amb aquest nom clau</p>
	 */
	public static final String CURRENT_MAV_KEY = "org.jewel.CURRENT_MAV";
	
}
