/**
 * 
 */
package org.jewel.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.jewel.core.ApplicationConfiguration;
import org.jewel.core.ApplicationController;
import org.jewel.core.RequestContext;
import org.jewel.exceptions.ParameterCastException;


/**
 * <p>Conjunt de mtodes tils a l'hora de tractar parmetres, ja siguin de context de Jewel
 * o de la pgina JSP</p>
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class ParameterUtil {

	public static String[] TRUE_PATTERNS = {"true", "y", "s"};
	
	public static boolean getBoolean(String param) {
		return getBoolean(param, false);
	}
	
	public static boolean getBoolean(String param, boolean defValue) {
		if (param == null) return defValue;
		for (String s : TRUE_PATTERNS) {
			if (s.equals(param)) return true;
		}
		return defValue;
	}

	public static int getInteger(String param) {
		return getInteger(param, 0);
	}
	
	public static int getInteger(String param, int defValue) {
		if (param == null) return defValue;
		try {
			return Integer.parseInt(param);
		} catch (Exception e) {
			return defValue;
		}
	}
	
	public static float getFloat(String param) {
		return getFloat(param, 0);
	}
	
	public static float getFloat(String param, float defValue) {
		if (param == null) return defValue;
		try {
			return Float.parseFloat(param);
		} catch (Exception e) {
			return defValue;
		}
	}

	/**
	 * <p>Converteix un objecte d'un parmetre a un tipus d'objecte marcat per el primer argument</p>
	 * <p>Pot convertir a Integer, String, Float, Boolean, Double, Long, Date o qualsevol
	 * array dels anteriors tipus. A ms, pot convertir a qualsevol altre tipus definit
	 * per el programador.</p>
	 * 
	 * @param type Tipus java.util.*, Arrays<java.util.*> o JavaBean personalitzats 
	 * @param value valor del parmetre (String o Object[] normalment)
	 * @return Objecte convertit al tipus especificat
	 * @throws ParameterCastException Error al convertir el parmetre al tipus indicat
	 */
	public static Object getParameterObject(Class<?> type, Object value) throws ParameterCastException {
		Object param = null;
		if (value != null) {
			try {
				if (value instanceof String && "".equals((String) value)) {
					return null;
				}
				
				// Converteix als tipus java.util.*
				if (type.equals(Integer.class)) {
					param = Integer.parseInt((String) value);
				}
				else if (type.equals(String.class)) {
					param = (String) value;
				}
				else if (type.equals(Float.class)) {
					param = Float.parseFloat((String) value);
				}
				else if (type.equals(Boolean.class)) {
					param = getBoolean((String) value, false);
				}
				else if (type.equals(Double.class)) {
					param = Double.parseDouble((String) value);
				}
				else if (type.equals(Long.class)) {
					param = Long.parseLong((String) value);
				}
				else if (type.equals(Date.class)) {
					//Totes les dates passades per parmetre es passen en milisegons
					param = new Date(Long.parseLong((String) value));
				}
				// Converts to array:
				else if (type.equals(Integer[].class)) {
					if (value instanceof Object[])
						param = ArrayUtil.toIntegerArray((Object[]) value);
					else
						param = new Integer[]{ Integer.parseInt((String) value) };
				}
				else if (type.equals(String[].class)) {
					if (value instanceof Object[])
						param = ArrayUtil.toStringArray((Object[]) value);
					else
						param = new String[]{ (String) value };
				}
				else if (type.equals(Float[].class)) {
					param = (Float[]) value;
				}
				else if (type.equals(Boolean[].class)) {
					if (value instanceof Object[]) {
						Object[] valueArray = (Object[]) value;
						Boolean[] booleanArray = new Boolean[valueArray.length];
						int cont = 0;
						for (Object o : valueArray) {
							booleanArray[cont++] = getBoolean((String) o, false);
						}
						param = booleanArray;
					}
					else {
						param = new Boolean[] { getBoolean((String) value, false) };
					}
				}
				else if (type.equals(Double[].class)) {
					if (value instanceof Object[]) {
						Object[] valueArray = (Object[]) value;
						Double[] doubleArray = new Double[valueArray.length];
						int cont = 0;
						for (Object o : valueArray) {
							doubleArray[cont++] = Double.parseDouble((String) o);
						}
						param = doubleArray;
					}
					else {
						param = new Double[] { Double.parseDouble((String) value) };
					}
				}
				else if (type.equals(Long[].class)) {
					if (value instanceof Object[]) {
						Object[] valueArray = (Object[]) value;
						Long[] longArray = new Long[valueArray.length];
						int cont = 0;
						for (Object o : valueArray) {
							longArray[cont++] = Long.parseLong((String) o);
						}
						param = longArray;
					}
					else {
						param = new Long[] { Long.parseLong((String) value) };
					}
				}
				else if (type.equals(Date[].class)) {
					if (value instanceof Object[]) {
						Object[] valueArray = (Object[]) value;
						Date[] dateArray = new Date[valueArray.length];
						int cont = 0;
						for (Object o : valueArray) {
							dateArray[cont++] = new Date(Long.parseLong((String) o));
						}
						param = dateArray;
					}
					else {
						param = new Date[] { new Date(Long.parseLong((String) value)) };
					}
				}
			}
			catch (NumberFormatException e) {
				throw new ParameterCastException("Error de format al convertir el parmetre '" + value + "' a " + type.getName());
			}
			catch (ClassCastException e) {
				throw new ParameterCastException("Error al convertir el parmetre '" + value + "' a tipus " + type.getName());
			}
		}
		
		return param;
	}

	/**
	 * <p>Retorna <i>true</i> si el tipus s Integer, String, Long, Float,
	 * Double, Date o un array dels anteriors</p> 
	 * @param type
	 * @return <i>true</i> si s de tipus Integer, String, Long, Float, Double, Date o un array dels anteriors
	 */
	public static boolean isJavaObject(Class<?> type) {
		return (type.equals(Integer.class) || type.equals(String.class) || type.equals(Long.class)
				|| type.equals(Float.class) || type.equals(Double.class) || type.equals(Date.class)
				|| type.equals(Integer[].class) || type.equals(String[].class) || type.equals(Long[].class)
				|| type.equals(Float[].class) || type.equals(Double[].class) || type.equals(Date[].class));
	}

	
	/**
	 * <p>Obt un objecte JavaBean del context, posant-hi les propietats que estiguin al context.</p>
	 * <p>Noms es posen les propietats de tipus Integer, String, Long, Float, Double, Date o un array
	 * dels anteriors tipus</p>
	 * 
	 * @param paramType Classe tipus de l'objecte que es cerca
	 * @param paramName Nom del parmetre cercat
	 * @param ctx Context on es cercar el parmetre
	 * @return Valor del parmetre
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 * @throws ParameterCastException 
	 */
	public static Object getParameterJavabean(Class<?> paramType,
			String paramName, RequestContext ctx) throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException, ParameterCastException {
		Object o = paramType.getConstructor().newInstance();
		
		for (Field field : paramType.getDeclaredFields()) {
			Object fieldValue = null;
			if (isJavaObject(field.getType())) {
				Object auxValue = ctx.getParameter(paramName + "." + field.getName());
				fieldValue = getParameterObject(field.getType(), auxValue);
			}
			else {
				fieldValue = getParameterJavabean(field.getType(), paramName + "." + field.getName(), ctx);
			}
			
			if (fieldValue != null) {
				BeanUtils.setProperty(o, field.getName(), fieldValue);
			}
		}
		return o;
	}
	
	
	/**
	 * <p>Obt la propietat d'un objecte pel seu nom, tenint en compte que si hi ha un punt
	 * accedir a la propietat que la cont successivament<p>
	 * <p>Per exemple, si <i>propertyName=persona.cotxe.matricula</i> s'accedir a la propietat
	 * 'matricula' de l'objecte cotxe, que es troba a l'objecte persona de l'objecte indicat al 
	 * primer parmetre</p>
	 * 
	 * @param bean Objecte on es cercar la propietat
	 * @param propertyName nom d'accs a la propietat
	 * @return Objecte que representa la propietat
	 * @throws NoSuchMethodException Si la propietat no s accessible
	 */
	public static Object getProperty(Object bean, String propertyName) throws NoSuchMethodException {
		Object property = bean;
		String[] elementsTree = propertyName.split("\\.");
		for (int i = 0; i < elementsTree.length; i ++) {
			if (property != null) {
				try {
					property = PropertyUtils.getProperty(property, elementsTree[i]);
				} catch (Exception e) {
					e.printStackTrace();
					throw new NoSuchMethodException("L'atribut '" + property + "' no t la propietat '" + elementsTree[i] + "' o no s accessible.");
				}
			}
		}
		return property;
	}
	
	
	/**
	 * Obt el resultat d'avaluar una expressi al context de la pgina, podent ser un objecte del context,
	 * un missatge internacionalitzat o, sino, es retorna l'expressi en si.
	 * 
	 * @param expression Expressi Jewel 
	 * @param pageContext Context de la pgina
	 * @return Valor de l'expressi
	 * @throws JspException Si s'ha produt un error cercant el valor al context de la pgina
	 */
	public static Object getExpressionValue(String expression, PageContext pageContext) throws JspException {
		if (PatternUtil.isJewelExpression(expression)) {
			//ApplicationConfiguration config = ApplicationConfiguration.getInstance();
			
			String elementsName = PatternUtil.getExpression(expression);
			Object bean = null;
			if (elementsName.contains(".")) {
				// Pot ser un atribut compost (o JavaBean) o una propietat d'un controlador de sessi
				String[] elementsChain = elementsName.split("\\.");
				
				String beanName = elementsChain[0];
				
				bean = pageContext.findAttribute(beanName);
				if (bean == null) {
					// Mirem si ho trobem a un controlador de sessi
					@SuppressWarnings("unchecked")
					Map<String, Object> sessionControllers = (Map<String, Object>) pageContext.getSession().getAttribute(JewelConstants.SESSION_CONTROLLERS_KEY);
					
					if (sessionControllers != null && sessionControllers.containsKey(beanName)) {
						bean = sessionControllers.get(beanName);
					}
					else {
						// Mirem si trobem el controlador de sessi al mapeig
						Class<?> controllerClass = ApplicationController.getInstance().getSessionControllerClass(beanName);
						if (controllerClass != null) {
							// Existeix un controlador de sessi declarat. L'instanciam i el guardem en sessi
							try {
								bean = controllerClass.newInstance();
								if (sessionControllers == null) {
									sessionControllers = new HashMap<String, Object>();
								}
								sessionControllers.put(beanName, bean);
								pageContext.getSession().setAttribute(JewelConstants.SESSION_CONTROLLERS_KEY, sessionControllers);
							} catch (Exception e) {
								e.printStackTrace();
							}
						}
					}
				}
				
				if (bean != null) {
					try {
						for (int i=1; i < elementsChain.length; i++) {
							bean = ParameterUtil.getProperty(bean, elementsChain[i]);
						}
					}
					catch (NoSuchMethodException e) {
						throw new JspException(e);
					}
				}
			}
			else {
				bean = pageContext.findAttribute(elementsName);
			}
			
			if (bean == null) bean = "";
			else if (bean instanceof Date) {
				// A la vista tractem amb els camps Date com a Long
				bean = ((Date) bean).getTime();
			}
			
			return bean;
		}
		else if (PatternUtil.isJewelMessageExpression(expression)) {
			ApplicationConfiguration config = ApplicationConfiguration.getInstance();
			ResourceBundle rb = (ResourceBundle) pageContext.getRequest().getAttribute(config.getLocaleConfig().getMessagesVariable());
			
			String value = PatternUtil.getMessageExpression(expression, rb);
			return (value == null ? "" : value);
		}
		return expression;
	}
	
}
