/**
 * 
 */
package org.jewel.util;

import java.util.Map;
import java.util.Map.Entry;

import org.jewel.core.ApplicationController;
import org.jewel.exceptions.InvalidPathPatternException;

/**
 * Utilitats per formar les URL de les accions
 * 
 * @author Juanma
 * @version 1.0
 *
 */
public class UrlUtil {

	/**
	 * Obt la URL que s'ha de cridar per executar una acci amb els parmetres indicats
	 * 
	 * @param ctxPath Direcci de context o direcci base per formar la URL
	 * @param action Acci a executar
	 * @param parametres Parmetres de l'acci
	 * @return URL ben formada per executar l'acci
	 * @throws InvalidPathPatternException Error produit si no existeix l'acci
	 */
	public static String getUrl(String ctxPath, String action, Map<String, String> parametres) throws InvalidPathPatternException {
		String url = ApplicationController.getInstance().getPattern(action);
		
		if (url == null) throw new InvalidPathPatternException("No existeix cap acci amb el patr " + action);
		
		String urlPattern = ApplicationController.getInstance().getServletUrlPattern();
		StringBuffer completeUrl = new StringBuffer(ctxPath);
		if (urlPattern.contains("/*")) {
			completeUrl.append(urlPattern.replace("/*", (url != null ? url : action)));
		}
		else {
			completeUrl.append(urlPattern.replace("*", (url != null ? url : action)));
		}
		
		String finalUrl = completeUrl.toString();
		if (parametres != null && parametres.size() > 0) {
			for (Entry<String, String> parametre : parametres.entrySet()) {
				finalUrl = finalUrl.replace("#{"+parametre.getKey()+"}", parametre.getValue());
			}
		}
		
		if (finalUrl.indexOf("#{") != -1) {
			throw new InvalidPathPatternException("La URL del bot est mal formada.");
		}
		
		return finalUrl;
	}
}
