/**
 * 
 */
package com.jewel.test.util;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.jewel.util.PatternUtil;
import org.junit.Test;

/**
 * @author Juanma
 *
 */
public class PatternUtilTest {

	/**
	 * Test method for {@link org.jewel.util.PatternUtil#validateActionPattern(java.lang.String)}.
	 */
	@Test
	public void testValidatePathDeclarationPattern() {
		// Valid patterns
		assertTrue(PatternUtil.validateActionPattern("/hola/1234/hola-que-tal"));
		assertTrue(PatternUtil.validateActionPattern("/aaa/parametro"));
		assertTrue(PatternUtil.validateActionPattern("/algo/1234/aaa/1234"));
		assertTrue(PatternUtil.validateActionPattern("/algo/#{parametro}"));
		assertTrue(PatternUtil.validateActionPattern("/hola/1234/#{parametro}"));
		assertTrue(PatternUtil.validateActionPattern("/hello/accio4/#{idMaquina2}"));
		assertTrue(PatternUtil.validateActionPattern("/hola/1234/#{parametro}/hola2/#{parametro}"));
		
		// Invalid path patterns
		assertFalse(PatternUtil.validateActionPattern("/hola/caracola/"));
		assertFalse(PatternUtil.validateActionPattern("/"));
		assertFalse(PatternUtil.validateActionPattern("/a$a"));
		assertFalse(PatternUtil.validateActionPattern("/!asl"));
		assertFalse(PatternUtil.validateActionPattern("hola"));
		assertFalse(PatternUtil.validateActionPattern("/&algo"));
		assertFalse(PatternUtil.validateActionPattern("1234"));
		assertFalse(PatternUtil.validateActionPattern("/"));
		assertFalse(PatternUtil.validateActionPattern("/hola"));
		assertFalse(PatternUtil.validateActionPattern("/#{hola}"));
		assertFalse(PatternUtil.validateActionPattern("hola"));
		assertFalse(PatternUtil.validateActionPattern("#{hola}"));
		assertFalse(PatternUtil.validateActionPattern("/hola/#{parametro}/"));
		assertFalse(PatternUtil.validateActionPattern("/hola/${parametro}"));
		assertFalse(PatternUtil.validateActionPattern("/#{parametro}"));
		
	}

	@Test
	public void testValidateExpressionPattern() {
		// Valid expression patterns
		List<String> s = PatternUtil.getExpressions("#{hello }");
		assertTrue(s.get(0).equals("hello"));
		
		s = PatternUtil.getExpressions("#{hello } #{hola} # uuu #{#{buah}");
		assertTrue(s.get(0).equals("hello"));
		assertTrue(s.get(1).equals("hola"));
		assertTrue(s.get(2).equals("#{buah"));
		// Invalid expression patterns
	}
	
	@Test
	public void testExpressionMatches() {
		// Valid expressions
		assertTrue(PatternUtil.urlMatches("/hola/#{para}", "/hola/pepe"));
		assertTrue(PatternUtil.urlMatches("/hola/#{para}", "/hola/a"));
		assertTrue(PatternUtil.urlMatches("/hola/#{para}", "/hola/1234"));
		assertTrue(PatternUtil.urlMatches("/hola/#{para}/", "/hola/pepe/"));
		assertTrue(PatternUtil.urlMatches("/hola/#{para}/", "/hola/1234/"));
		
		assertTrue(PatternUtil.urlMatches("/hola/#{para}/#{param2}", "/hola/1234/hola-que-tal"));
		
		assertFalse(PatternUtil.urlMatches("/hola/#{para}/", "/hola/pepe"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}", "/hola/pepe/"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}", "/hola"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}/", "/hola/"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}", "/hola/"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}/", "/hola"));
		assertFalse(PatternUtil.urlMatches("/hola/#{para}", "/hola/ "));
		
		assertTrue(PatternUtil.urlMatches("/accio4/${id}", "/hello/accio4/415"));
	}
	
	
	@Test
	public void testIsEL() {
		// valid patterns
		assertTrue(PatternUtil.isJewelExpression("#{hola}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola.pepe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola.pepe.repe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola }"));
		assertTrue(PatternUtil.isJewelExpression("#{ hola }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola   }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola.pepe.repe   }"));
		// valid array patterns
		assertTrue(PatternUtil.isJewelExpression("#{hola[1]}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola[1].pepe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola[1].pepe.repe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola[12] }"));
		assertTrue(PatternUtil.isJewelExpression("#{ hola[123] }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola.pepe[12]   }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola.pepe.repe[123]   }"));
		assertTrue(PatternUtil.isJewelExpression("#{hola['a1']}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola['1asdf'].pepe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola['1sdf_'].pepe.repe}"));
		assertTrue(PatternUtil.isJewelExpression("#{hola['12_asdf'] }"));
		assertTrue(PatternUtil.isJewelExpression("#{ hola['asdf123'] }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola.pepe['as_12']   }"));
		assertTrue(PatternUtil.isJewelExpression("#{  hola.pepe.repe['asdf123']   }"));
		// Invalid patterns
		assertFalse(PatternUtil.isJewelExpression("#{hola"));
		assertFalse(PatternUtil.isJewelExpression("#hola}"));
		assertFalse(PatternUtil.isJewelExpression("{hola}"));
		assertFalse(PatternUtil.isJewelExpression("hola"));
		assertFalse(PatternUtil.isJewelExpression("#hola"));
	}
}
