/**
 * 
 */
package org.jewel.example.action;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jewel.annotations.Action;
import org.jewel.annotations.Controller;
import org.jewel.annotations.ControllerScope;
import org.jewel.annotations.RolesAllowed;
import org.jewel.core.ActionMessages;
import org.jewel.core.ModelAndView;
import org.jewel.core.dispatchers.JsonResultDispatcher;
import org.jewel.core.dispatchers.TilesResultDispatcher;
import org.jewel.example.persistence.Departament;
import org.jewel.example.persistence.Empleat;
import org.jewel.example.persistence.Habilitat;
import org.jewel.example.persistence.PerfilEmpleat;
import org.jewel.example.services.DepartamentsService;
import org.jewel.example.services.EmpleatsService;
import org.jewel.example.services.HabilitatsService;
import org.jewel.example.services.PerfilsEmpleatService;

/**
 * @author Juanma
 *
 */
@Controller(name="adminEmpleats", path="/admin/empleats", scope=ControllerScope.REQUEST)
public class AdminEmpleatsController {

	@Action(path="/nou", params={})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView nouEmpleat() {
		Map<String, Object> model = getEmpleatModel();
		
		return (new ModelAndView(model, "empleat-nou", TilesResultDispatcher.class, null));
	}
	
	@Action(path="/crear", params={"codi", "nom", "cognom1", "cognom2", "idDept", "idPerfil", "idHabilitats"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView crearEmpleat(String codi, String nom, String cognom1, String cognom2, Integer idDept, Integer idPerfil, Integer[] idHabilitats) {
		// Guardem l'empleat
		EmpleatsService empleatsSrv = new EmpleatsService();
		if (!empleatsSrv.existeixCodiUsuari(codi)) {
			Empleat empleat = empleatsSrv.save(codi, nom, cognom1, cognom2, idDept, idPerfil, idHabilitats);
			
			// Si no hi ha errors, retornem al llistat
			return editarEmpleat(empleat.getId(), null, null, null, null, null, null, null);
		}
		else {
			Map<String, Object> model = getEmpleatModel();
			
			ActionMessages messages = new ActionMessages();
			messages.addErrorMessage("Aquest codi ja existeix");
			
			return (new ModelAndView(model, "empleat-nou", TilesResultDispatcher.class, messages));
		}
	}
	
	private Map<String, Object> getEmpleatModel() {
		DepartamentsService departamentsSrv = new DepartamentsService();
		List<Departament> departaments = departamentsSrv.findAll();
		
		PerfilsEmpleatService perfilsSrv = new PerfilsEmpleatService();
		List<PerfilEmpleat> perfils = perfilsSrv.findAll();
		
		HabilitatsService habilitatsSrv = new HabilitatsService();
		List<Habilitat> habilitats = habilitatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("departaments", departaments);
		model.put("perfils", perfils);
		model.put("habilitats", habilitats);
		
		return model;
	}
	
	
	@Action(path="/editar/#{id}", params={"clau", "nom", "cognom1", "cognom2", "idDept", "idPerfil", "idHabilitats"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView editarEmpleat(Integer idEmpleat, String clau, String nom, String cognom1, String cognom2, Integer idDept, Integer idPerfil, Integer[] idHabilitats) {
		EmpleatsService empleatsSrv = new EmpleatsService();
		DepartamentsService departamentsSrv = new DepartamentsService();
		PerfilsEmpleatService perfilsSrv = new PerfilsEmpleatService();
		
		Empleat empleat;
		if (nom != null) {
			empleat = empleatsSrv.update(idEmpleat, clau, nom, cognom1, cognom2, idDept, idPerfil, idHabilitats);
		}
		else {
			empleat = empleatsSrv.findById(idEmpleat);
		}
		
		List<Departament> departaments = departamentsSrv.findAll();
		List<PerfilEmpleat> perfils = perfilsSrv.findAll();
		
		HabilitatsService habilitatsSrv = new HabilitatsService();
		List<Habilitat> habilitats = habilitatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("empleat", empleat);
		model.put("departaments", departaments);
		model.put("perfils", perfils);
		model.put("habilitats", habilitats);
		
		return (new ModelAndView(model, "empleat-editar", TilesResultDispatcher.class, null));
	}
	
	@Action(path="/json/comprova-codi", params={"codi"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView comprobarCodiJson(String codi) {
		EmpleatsService empleatsSrv = new EmpleatsService();
		boolean check = empleatsSrv.existeixCodiUsuari(codi);
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("existeix", check);
		
		return (new ModelAndView(model, null, JsonResultDispatcher.class, null));
	}
}
