/**
 * 
 */
package org.jewel.example.action;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jewel.annotations.Action;
import org.jewel.annotations.Controller;
import org.jewel.annotations.ControllerScope;
import org.jewel.annotations.RolesAllowed;
import org.jewel.core.ModelAndView;
import org.jewel.core.dispatchers.TilesResultDispatcher;
import org.jewel.example.persistence.Empleat;
import org.jewel.example.persistence.Projecte;
import org.jewel.example.persistence.ProjecteEmpleat;
import org.jewel.example.services.EmpleatsService;
import org.jewel.example.services.ProjectesService;

/**
 * @author Juanma
 *
 */
@Controller(name="adminProjectes", path="/admin/projectes", scope=ControllerScope.REQUEST)
public class AdminProjectesController {

	@Action(path="/nou", params={})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView nouProjecte() {
		EmpleatsService empleatsSrv = new EmpleatsService();
		List<Empleat> empleats = empleatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("empleats", empleats);
		
		return (new ModelAndView(model, "projecte-nou", TilesResultDispatcher.class, null));
	}
	
	@Action(path="/crear", params={"codi", "nom", "idEmpleat", "dataObertura", "dataTancament"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView crearProjecte(String codi, String nom, Integer idEmpleat, Date dataObertura, Date dataTancament) {
		// Guardem el projecte
		ProjectesService projectesSrv = new ProjectesService();
		Projecte projecte = projectesSrv.save(codi, nom, idEmpleat, dataObertura, dataTancament);
		
		// Si no hi ha errors, retornem al llistat departaments
		return editarProjecte(projecte.getId(), null, null, null, null);
	}

	
	@Action(path="/editar/#{id}", params={"nom", "idEmpleat", "dataObertura", "dataTancament"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView editarProjecte(Integer idProjecte, String nom, Integer idEmpleat, Date dataObertura, Date dataTancament) {
		ProjectesService projectesSrv = new ProjectesService();
		EmpleatsService empleatsSrv = new EmpleatsService();
		
		Projecte projecte = null;
		if (nom != null) {
			projecte = projectesSrv.update(idProjecte, nom, idEmpleat, dataObertura, dataTancament);
		}
		else {
			projecte = projectesSrv.findById(idProjecte);
		}
		
		List<Empleat> empleats = empleatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("projecte", projecte);
		model.put("empleats", empleats);
		
		return (new ModelAndView(model, "projecte-editar", TilesResultDispatcher.class, null));
	}
	
	@Action(path="/#{id}/edita-empleat/#{idProjEmp}", params={})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView editarProjEmpleat(Integer idProjecte, Integer idProjEmp) {
		ProjectesService projectesSrv = new ProjectesService();
		EmpleatsService empleatsSrv = new EmpleatsService();
		
		Projecte projecte = projectesSrv.findById(idProjecte);
		ProjecteEmpleat projEmp = empleatsSrv.getProjecteEmpleat(idProjEmp);
		List<Empleat> empleats = empleatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("projecte", projecte);
		model.put("empleats", empleats);
		model.put("projEmp", projEmp);
		
		return (new ModelAndView(model, "projecte-editar-empleat", TilesResultDispatcher.class, null));
	}
	
	@Action(path="/#{id}/guardar-empleat/#{idProjEmp}", params={"dataInici", "dataFi", "valoracio"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView guardarEmpleat(Integer idProjecte, Integer idProjEmp, Date dataInici, Date dataFi, Float valoracio) {
		if (dataInici != null) {
			EmpleatsService empleatsSrv = new EmpleatsService();
			
			empleatsSrv.updateProjecteEmpleat(idProjEmp, dataInici, dataFi, valoracio);
		}
		return editarProjecte(idProjecte, null, null, null, null);
	}
	
	@Action(path="/#{id}/nou-empleat", params={"idEmpleat", "dataInici", "dataFi"})
	@RolesAllowed(roles="jewel_admin")
	public ModelAndView afegirEmpleat(Integer idProjecte, Integer idEmpleat, Date dataInici, Date dataFi) {
		ProjectesService projectesSrv = new ProjectesService();
		EmpleatsService empleatsSrv = new EmpleatsService();
		
		Projecte projecte = projectesSrv.findById(idProjecte);
		// Afegim l'empleat al projecte
		if (idEmpleat != null && idEmpleat > 0 && dataInici != null) {
			empleatsSrv.assignarProjecte(projecte, idEmpleat, dataInici, dataFi);
		}
		
		List<Empleat> empleats = empleatsSrv.findAll();
		
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("projecte", projecte);
		model.put("empleats", empleats);
		
		return (new ModelAndView(model, "projecte-editar", TilesResultDispatcher.class, null));
	}
	
}
