package org.jewel.example.persistence;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;


/**
 * The persistent class for the empleat database table.
 * 
 */
@Entity
@Table(name="empleat")
public class Empleat implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	@Column(name="id_empleat", updatable=false, unique=true, nullable=false)
	private Integer id;

	@Column(nullable=false)
	private boolean baixa;

	@Column(name="codi_usuari", nullable=false, length=12)
	private String codiUsuari;
	
	@Column(nullable=false, length=30)
	private String cognom1;

	@Column(length=30)
	private String cognom2;

	@Column(length=30)
	private String email;

	@Column(nullable=false, length=30)
	private String nom;

	//bi-directional many-to-one association to Departament
    @ManyToOne
	@JoinColumn(name="perfil_id", nullable=false)
	private PerfilEmpleat perfil;
    
	//bi-directional many-to-one association to Departament
    @ManyToOne
	@JoinColumn(name="departament_id", nullable=false)
	private Departament departament;

	//bi-directional many-to-one association to ProjecteEmpleat
	@OneToMany(mappedBy="empleat")
	private List<ProjecteEmpleat> projecteEmpleats;
	
	@ManyToMany()
	@JoinTable(name="habilitat_empleat", 
		joinColumns={@JoinColumn(name="empleat_id")},
		inverseJoinColumns={@JoinColumn(name="habilitat_id")})
	private List<Habilitat> habilitats;

    public Empleat() {
    }

	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public boolean getBaixa() {
		return this.baixa;
	}

	public void setBaixa(boolean baixa) {
		this.baixa = baixa;
	}

	public String getCodiUsuari() {
		return this.codiUsuari;
	}

	public void setCodiUsuari(String codiUsuari) {
		this.codiUsuari = codiUsuari;
	}
	
	public String getCognom1() {
		return this.cognom1;
	}

	public void setCognom1(String cognom1) {
		this.cognom1 = cognom1;
	}

	public String getCognom2() {
		return this.cognom2;
	}

	public void setCognom2(String cognom2) {
		this.cognom2 = cognom2;
	}

	public String getEmail() {
		return this.email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getNom() {
		return this.nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public Departament getDepartament() {
		return this.departament;
	}

	public void setDepartament(Departament departament) {
		this.departament = departament;
	}
	
	public PerfilEmpleat getPerfil() {
		return perfil;
	}
	
	public void setPerfil(PerfilEmpleat perfil) {
		this.perfil = perfil;
	}
	
	public List<ProjecteEmpleat> getProjecteEmpleats() {
		return this.projecteEmpleats;
	}

	public void setProjecteEmpleats(List<ProjecteEmpleat> projecteEmpleats) {
		this.projecteEmpleats = projecteEmpleats;
	}
	
	public List<Habilitat> getHabilitats() {
		return habilitats;
	}
	
	public void setHabilitats(List<Habilitat> habilitats) {
		this.habilitats = habilitats;
	}
	
}