/**
 * 
 */
package org.jewel.example.services;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Session;
import org.jewel.example.javabean.FiltreEmpleats;
import org.jewel.example.persistence.Empleat;
import org.jewel.example.persistence.Departament;
import org.jewel.example.persistence.Habilitat;
import org.jewel.example.persistence.PerfilEmpleat;
import org.jewel.example.persistence.Projecte;
import org.jewel.example.persistence.ProjecteEmpleat;
import org.jewel.example.services.dao.EmpleatsDAO;
import org.jewel.example.services.dao.DepartamentsDAO;
import org.jewel.example.services.dao.HabilitatsDAO;
import org.jewel.example.services.dao.HibernateUtil;
import org.jewel.example.services.dao.PerfilsEmpleatDAO;
import org.jewel.example.services.dao.ProjecteEmpleatsDAO;

/**
 * @author Juanma
 *
 */
public class EmpleatsService {
	
	public List<Empleat> findAll() {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		
		return empleatsDAO.findAll();
	}
	
	public Empleat save(String codi, String nom, String cognom1, String cognom2, Integer idDepartament, Integer idPerfil, Integer[] idHabilitats) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		DepartamentsDAO departamentsDAO = new DepartamentsDAO(session);
		PerfilsEmpleatDAO perfilsDAO = new PerfilsEmpleatDAO(session);
		HabilitatsDAO habilitatsDAO = new HabilitatsDAO(session);
		
		Empleat empleat = new Empleat();
		empleat.setCodiUsuari(codi);
		empleat.setNom(nom);
		empleat.setCognom1(cognom1);
		empleat.setCognom2(cognom2);
		if (idDepartament != null) {
			Departament departament = departamentsDAO.findById(idDepartament);
			empleat.setDepartament(departament);
		}
		if (idPerfil != null) {
			PerfilEmpleat perfil = perfilsDAO.findById(idPerfil);
			empleat.setPerfil(perfil);
		}
		if (idHabilitats != null) {
			List<Habilitat> habilitats = new ArrayList<Habilitat>();
			for (Integer idHabilitat : idHabilitats) {
				Habilitat habilitat = habilitatsDAO.findById(idHabilitat);
				habilitats.add(habilitat);
			}
			empleat.setHabilitats(habilitats);
		}
		
		empleat = empleatsDAO.create(empleat);
		empleatsDAO.flush();
		
		return empleat;
	}

	public List<Empleat> findByFiltre(FiltreEmpleats filtre) {
		if (filtre == null) return findAll();
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		
		return empleatsDAO.findByFiltre(filtre);
	}

	public Empleat update(Integer idEmpleat, String clau, String nom, String cognom1, String cognom2, Integer idDepartament, Integer idPerfil, Integer[] idHabilitats) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		DepartamentsDAO departamentsDAO = new DepartamentsDAO(session);
		PerfilsEmpleatDAO perfilsDAO = new PerfilsEmpleatDAO(session);
		HabilitatsDAO habilitatsDAO = new HabilitatsDAO(session);
		
		Empleat empleat = empleatsDAO.findById(idEmpleat);
		if (empleat != null && nom != null) {
			empleat.setNom(nom);
			empleat.setCognom1(cognom1);
			empleat.setCognom2(cognom2);
			if (idDepartament != null) {
				Departament departament = departamentsDAO.findById(idDepartament);
				empleat.setDepartament(departament);
			}
			if (idPerfil != null) {
				PerfilEmpleat perfil = perfilsDAO.findById(idPerfil);
				empleat.setPerfil(perfil);
			}
			if (idHabilitats != null) {
				List<Habilitat> habilitats = new ArrayList<Habilitat>();
				for (Integer idHabilitat : idHabilitats) {
					Habilitat habilitat = habilitatsDAO.findById(idHabilitat);
					habilitats.add(habilitat);
				}
				empleat.setHabilitats(habilitats);
			}
			
			empleatsDAO.update(empleat);
		}
		empleatsDAO.flush();
		
		return empleat;
	}

	public Empleat findById(Integer idEmpleat) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		
		return empleatsDAO.findById(idEmpleat);
	}

	public Empleat findByCodi(String codi) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		
		return empleatsDAO.findByCodi(codi);
	}

	public boolean existeixCodiUsuari(String codi) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		Empleat empleat = empleatsDAO.findByCodi(codi);
		
		return (empleat != null);
	}

	public void assignarProjecte(Projecte projecte, Integer idEmpleat,
			Date dataInici, Date dataFi) {
		Session session = HibernateUtil.getSession();
		EmpleatsDAO empleatsDAO = new EmpleatsDAO(session);
		ProjecteEmpleatsDAO projEmpsDAO = new ProjecteEmpleatsDAO(session);
		
		Empleat empleat = empleatsDAO.findById(idEmpleat);
		
		ProjecteEmpleat projEmp = new ProjecteEmpleat();
		projEmp.setProjecte(projecte);
		projEmp.setEmpleat(empleat);
		projEmp.setDataInici(dataInici);
		projEmp.setDataFi(dataFi);
		projEmp.setValoracio(null);
		projEmp.setPerfil(empleat.getPerfil());
		
		projEmpsDAO.create(projEmp);
	}

	
	public ProjecteEmpleat getProjecteEmpleat(Integer idProjEmp) {
		Session session = HibernateUtil.getSession();
		ProjecteEmpleatsDAO projEmpsDAO = new ProjecteEmpleatsDAO(session);
		return projEmpsDAO.findById(idProjEmp);
	}

	public void updateProjecteEmpleat(Integer idProjEmp, Date dataInici,
			Date dataFi, Float valoracio) {
		Session session = HibernateUtil.getSession();
		ProjecteEmpleatsDAO projEmpsDAO = new ProjecteEmpleatsDAO(session);
		ProjecteEmpleat projEmp = projEmpsDAO.findById(idProjEmp);
		projEmp.setDataInici(dataInici);
		projEmp.setDataFi(dataFi);
		projEmp.setValoracio(valoracio);
		
		projEmpsDAO.update(projEmp);
		projEmpsDAO.flush();
	}
	
}
