/**
 * 
 */
package org.jewel.example.services.dao;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;


/**
 * @author Juanma
 * 
 */
public abstract class AbstractHibernateDAO<MODEL, ID extends Serializable>
		implements InterfaceHibernateDAO<MODEL, ID> {

	private Class<MODEL> persistentClass;
	private Session session;

	@SuppressWarnings("unchecked")
	public AbstractHibernateDAO(Session session) {
		this.persistentClass = (Class<MODEL>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
		this.session = session;
	}
/*
	public void setSession(Session s) {
		this.session = s;
	}
*/
	protected Session getSession() {
		if (session == null)
			throw new IllegalStateException(
					"Session has not been set on DAO before usage");
		return session;
	}

	public Class<MODEL> getPersistentClass() {
		return persistentClass;
	}

	@Override
	public MODEL findById(ID id) {
		@SuppressWarnings("unchecked")
		MODEL entity = (MODEL) getSession().load(getPersistentClass(), id);

		return entity;
	}
	
	@Override
	public List<MODEL> findAll() {
		return findByCriteria();
	}

	@Override
	public MODEL create(MODEL entity) {
		getSession().save(entity);
		return entity;
	}

	@Override
	public MODEL update(MODEL entity) {
		getSession().update(entity);
		return entity;
	}
	
	@Override
	public void delete(MODEL entity) {
		getSession().delete(entity);
	}

	public void flush() {
		getSession().flush();
	}

	public void clear() {
		getSession().clear();
	}

	/**
	 * Use this inside subclasses as a convenience method.
	 */
	@SuppressWarnings("unchecked")
	protected List<MODEL> findByCriteria(Criterion... criterion) {
		Criteria crit = getSession().createCriteria(getPersistentClass());
		for (Criterion c : criterion) {
			crit.add(c);
		}
		return crit.list();
	}
}
